// |$Id: bequ.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: bequ.c                                      |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: Trace equalization of seismic traces              |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/c_bsegy.h"

//...parameter definitions

//...function prototypes
      #include "../include/sub4.h"

      int  getparm(int argc,char *argv[],
        float *tmin1, float *tmax1, int *normsel, char infile[],
        char ofile[], char lstfile[], char ampfile[], char pid[]);

      void outlst(float tmin1, float tmax1, int normsel, char infile[],
        char pid[], char ofile[], char lstfile[], char ampfile[],
        FILE *h3);

      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h2, *h3;
      int               jt,
                        jrec,
                        npts,
                        ntraces,
                        ibar,
                        invbar;
      float             fsamin,
                        bar,
                        *s1,
                        *s2;
      char              pid[5]="bequ",   //process id name
                        ofile[13],
                        lstfile[13],
                        infile[14];
      struct header     hd;

//...specific to this application
      FILE             *h4;
      float             tmin1,
                        tmax1,
                        tmax11,
                        fleng,
                        sum;
      char              ampfile[13];
      int               istart,
			normsel,
                        istop;

//...get parameters
      if(getparm(argc,argv,&tmin1,
         &tmax1,&normsel,infile,ofile,lstfile,ampfile,pid)!=0)
      {
       goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...echo check to listing file
      outlst(tmin1,tmax1,normsel,infile,pid,ofile,lstfile,ampfile,h3);

//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",infile);
          goto exit_3;
        }

//...open output file
        if( (h2=fopen(ofile,"wb"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",ofile);
          goto exit_1;
        }

//...open output file
        if( (h4=fopen(ampfile,"w"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",ofile);
          goto exit_2;
        }


//...check input file size
          if(in_chk(&ntraces,&npts,&fsamin,hd,h1)!=0)
         {
          goto exit_4;
         }

//...print input file data to lstfile
        fprintf(h3,"\n     Number of samples= %d",npts);
        fprintf(h3,"\n     Sample Interval = %5.4f sec.",fsamin);
        fprintf(h3,"\n     Number of traces = %d",ntraces);

//...parameter test/reset valid values
        tmax11=(float)(npts-1)*fsamin;
        if(tmin1<0.0) tmin1=0.0;
        if(tmax1>tmax11) tmax1=tmax11;
        istart=(int)(tmin1/fsamin+0.5);
        istop=(int)(tmax1/fsamin+0.5);
        if(istart<0) istart=0;
        if(istop>npts) istop=npts;
        tmin1=(float)(istart)*fsamin;
        tmax1=(float)(istop)*fsamin;
        fleng=(float)(istop-istart);
        printf("\n  tmin=%7.4f  tmax=%7.4f",tmin1,tmax1);

//...dynamic allocation of seimic trace memory
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          puts("ABORT:s1 can't allocate enough trace memory");
          printf(" npts = %d",npts);
          goto exit_4;
         }
        s2=calloc(npts,sizeof(float));
          if(s2==NULL)
         {
          puts("ABORT:s2 can't allocate enough trace memory");
          printf(" npts = %d",npts);
          goto exit_s1;
         }
//-----------------main loop---------------------------
//...display progress bar grid

      bargrid(ntraces,&bar,&ibar,&invbar);

	if(normsel==2)
      fprintf(h3,"\n    Trace  RMS Value"); //listing col. headers
	else
      fprintf(h3,"\n    Trace  Peak abs(Value)"); //listing col. headers

//...trace loop

      for (jrec=0;jrec<ntraces;jrec++)
     {

//...read trace
        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        puts("ABORT--c_bsegin");
        goto quit_it;
       }
//...select normalization method
	if (normsel==2)
       {
//...compute L2 norm and scale trace...
         sum=0.0;
         for (jt=istart;jt<istop;jt++) sum=s1[jt]*s1[jt]+sum;
         if(sum!=0.0) sum=sqrt(sum/fleng); else sum=1.0;
       }  
         else
       {
	 sum=-999999999.;
         for (jt=istart;jt<istop;jt++) 
	   {
	    if(fabs(s1[jt])>sum)  sum=fabs(s1[jt]);
	   } //next jt
            if (sum==0.0) sum=1.0;
       }  //endif normsel
         fprintf(h3,"\n    %5.5d   %e",jrec,sum);
         fprintf(h4,"\n %5.5d %e",jrec+1,sum);  //output scale factor

//...apply scale factor
        for(jt=0;jt<npts;jt++) s2[jt]=s1[jt]/sum;


//...output trace
        if(c_bsegout(jrec,s2,npts,&hd,h2)!=0)
       {
        puts("ABORT--c_bsegout");
        goto quit_it;
       }

//...display progress bar
      exbar(ntraces,bar,ibar,invbar,jrec+1);

     } //  end trace loop-----------------------------

      printf("\n   Listing ====>%s",lstfile);
      printf("\n   Output  ====>%s",ofile);
      printf("\n   Amplitudes==>%s",ampfile);
      printf("\n   Processed %d traces",ntraces);
	return(0);
quit_it:
        puts(" ");

        free(s2);
exit_s1:
        free(s1);

exit_4:
        fclose(h4);
exit_2:
        fclose(h2);
exit_1:
        fclose(h1);
exit_3:
        fclose(h3);

quick_exit:
	return(1);

     } //  end of main

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
         float *tmin1, float *tmax1, int *normsel, char infile[],
         char ofile[], char lstfile[],char ampfile[], char pid[])
     {
      char              sfx1[4]="seg",
                        sfx2[4]="lst",
                        sfx3[4]="dat";
      char		help[3]="-h";
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"Trace Equalization by L2 Norm                ");
      sprintf(msg2,"   or by Peak Absolute Value                 ");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  %s  infile     tmin   tmax   normsel ",pid);
      puts(" ");
      puts("     infile   =  input file name");
      puts("     tmin     =  gate: minimum time (s) ");
      puts("     tmax     =  gate: maximum time (s) ");
      puts("     normsel  =  select normalization ");
      puts("             2=  L2 Norm   ");
      puts("             0=  Peak abs(Value) ");
      puts(" ");
      puts("  NOTE: Default is normsel=2 ");
      puts("  No interactive prompt for normsel");
      puts("  (Must be specified on command line) ");
      puts(" ");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strncpy(infile,argv[1],13);
       else
      {
       puts(" Enter INPUT file name ");
       scanf(" %13s",infile);
      }
	if (namecheck(12,infile) != 0 ) return (1);

//...convert string to integer
      if(argc>2)
       *tmin1=atof(argv[2]);
       else
      {
       puts(" Enter tmin (s)  (float)");
       scanf(" %f",tmin1);
      }

//...convert string to float
      if(argc>3)
       *tmax1=atof(argv[3]);
       else
      {
       puts(" Enter tmax (s) (float)");
       scanf(" %f",tmax1);
      }

//...convert string to float
      if(argc>4)
       *normsel=atoi(argv[4]);
       else
      {
       *normsel=2;
//       puts(" Enter normsel (int)");
//       scanf(" %d",normsel);
      }

//...name output files
      names(infile,pid,ofile,sfx1);
      names(infile,pid,lstfile,sfx2);
      names(infile,pid,ampfile,sfx3);

       return(0);
     }  //  endif bhelp test
      } // end getparm

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(float tmin1, float tmax1, int normsel, char infile[],
        char pid[], char ofile[], char lstfile[],char ampfile[],
        FILE *h3)
     {

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %s.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Input   File= %s",infile);
      fprintf(h3,"\n     Output  File= %s",ofile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fprintf(h3,"\n     Amplitudes File= %s",ampfile);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fprintf(h3,"\n     tmin= %7.4f ",tmin1);
      fprintf(h3,"\n     tmax= %7.4f ",tmax1);
      fprintf(h3,"\n     normsel= %d ",normsel);
      fputs("\n",h3);                      //blank line
     }  //  end outlst
