// |$Id: bgar.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: bgar.c                                      |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: bgar determines exponential gain and applies      |
   |          the correction to the entire data set.            |
   |                                                            |
   | User restricts the offset range for the decay measurement  |
   | (Designed for surface surveys)                             |
   | Ampitude measured is maximum Peak-Peak direct arrival      |
   |                                                            |
   | The difference between bgaz.c and bgar.c is largely in the |
   | specification of the gate within which amplitude analysis  |
   | is conducted (depth vs range specification).  In both cases|
   | range beyond the reference controls the application of the |
   | gain recovery.  The other difference is in the way the     |
   | amplitude envelope is computed (peak-peak maximum in bgaz, |
   | sum of absolute values in each trace, bgar).  If one       |
   | prefers this latter estimate in a down-hole survey, then   |
   | bgar could be run on down-hole data, the only inconvenience|
   | being a slight depth <--> range conversion is specifying   |
   | the gate.                                                  |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |REQUIRES PLPLOT:                                            |
   |PLPLOT is a GNU plot library which supports both C and      |
   |Fortran calls.  The distribution may be downloaded from     |
   |http://plplot.sourceforge.net                               |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |  amplot     plots least squares fit to dB decay            |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |  lsqufit    least squares fit, x,y data                    |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <ctype.h>
      #include <time.h>
      #include "../include/c_bsegy.h"
#ifdef PLPLOT
      #include "plConfig.h"
      #include "plplot.h"
#endif
//...parameter definitions
      #define  maxtraces 500

//...function prototypes
      #include "../include/sub4.h"

      int  getparm(int argc,char *argv[],
        float *rmin, float *rmax, float *dbu, char infile[],
        char ofile[], char lstfile[], char pid[]);

      void outlst(float rmin, float rmax, float dbu, char infile[],
        char pid[], char ofile[], char lstfile[],
        FILE *h3);

#ifdef PLPLOT
      int amplot(PLINT npts, float x[], float y[],  float yc[],
		     float slope,  float bcept,  float r0 );
#else
      void amplot(int npts, float x[], float y[],
		     float slope,  float bcept,  float r0 );
#endif


      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h2, *h3;
      int               jt,
                        jrec,
                        npts,
                        ntraces,
                        ibar,
                        invbar;
      float             fsamin,
                        bar,
                        *s1,
                        *s2;
      char              pid[5]="bgar",   //process id name
                        ofile[13],
                        lstfile[13],
                        infile[14];
      struct header     hd;

//...specific to this application
      float             rmin,            //minimum range of gate
                        rmax,            //maximum range of gate
                        dbu;             //exponential correction dB/m
      float             sx, sy, sz, sd;  //source coordinates
      float             rx, ry, rz;      //receiver coordinates
      float             range;           //source-receiver separation
                                         //stored geometry info
      float             rangexyz[maxtraces];
      float		x[maxtraces],    //single precision, BSU functions
			y[maxtraces],
			yc[maxtraces];
      double		tmp1,tmp2;

      float             envlp[maxtraces];  //envelope signal
      float             maxr,minr;       //actual max,min range in data
      float		r0;              //reference offset
      float		envlp0;		 //envelope at reference offset
      float		neper;

      float             slope,intrcpt,lsqe,r2,gainf;

#ifdef PLPLOT
      PLINT             kpts,            //number samples in fit
      			k,kr0;
#else
      int               kpts,            //number samples in fit
      			k,kr0;
#endif
      int		kmx,kmn;	 //indexes for max and min values


//...get parameters
      if(getparm(argc,argv,&rmin,&rmax,&dbu,
        infile,ofile,lstfile,pid)!=0)
      {
       goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...echo check to listing file


      outlst(rmin,rmax,dbu,infile,pid,ofile,lstfile,h3);

//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",infile);
          goto exit_3;
        }

//...open output file
        if( (h2=fopen(ofile,"wb"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",ofile);
          goto exit_1;
        }

//...check input file size
          if(in_chk(&ntraces,&npts,&fsamin,hd,h1)!=0)
         {
          goto exit_2;
         }

//...print input file data to lstfile
        fprintf(h3,"\n     Number of samples= %d",npts);
        fprintf(h3,"\n     Sample Interval = %5.4f sec.",fsamin);
        fprintf(h3,"\n     Number of traces = %d",ntraces);

//...check number traces
        if(ntraces>maxtraces)
        {
         puts(" ABORT  !!! number of traces exceeds dimension");
         goto exit_2;
        }

//...dynamic allocation of seimic trace memory
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          puts("ABORT:s1 can't allocate enough trace memory");
          printf(" npts = %d",npts);
          goto exit_2;
         }
        s2=calloc(npts,sizeof(float));
          if(s2==NULL)
         {
          puts("ABORT:s2 can't allocate enough trace memory");
          printf(" npts = %d",npts);
          goto exit_s1;
         }

//---------------------offset range check, list----------------
//...header for geometry listing
      fputs("\n\n Receiver Coordinates Relative to Source:",h3);
      fputs("\n trace  delta_x   delta_y  ",h3);
      fputs("depth      rz     range\n",h3);

      for (jrec=0;jrec<ntraces;jrec++)
     {
//...read trace
        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        puts("ABORT--c_bsegin");
        goto quit_it;
       }

//...determine shot and receiver coordinates
      findxyz(&sx,&sy,&sz,&sd,&rx,&ry,&rz,&range,hd);

//...print coordinates to listing
      fprintf(h3,"\n  %3.3d %7.2f   %7.2f  %7.2f   %7.2f %7.2f",
        jrec+1,(rx-sx),(ry-sy),fabs(rz-sz),rz,range);

//...store geometry data in arrays
      rangexyz[jrec]=range;
     } //endfor

//...find reference offset
      fmax_min(ntraces,rangexyz,&maxr,&minr,1,&kmx,&kmn);
      r0=rangexyz[kmn];
      kr0=kmn;
      fprintf(h3,"\n Reference Offset, R0=%7.2f",r0);
      fprintf(h3,"\n Reference Trace Number=%3d",kmn+1);
      //note: Trace number is like in Fortran for listing, >=1
      //(for compatibility with Fortran codes)


//...max/min check on rmin, rmax available in data set
      if(fabs(rmax)>maxr) rmax=maxr;
      if(fabs(rmin)<minr) rmin=minr;
//----------------------end depth range check------------------

//----------------------compute envelope-----------------------
      kpts=0;

      for(jrec=0;jrec<ntraces;jrec++)
     {
       //...read trace
       if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
         {
          puts("ABORT--c_bsegin");
          goto quit_it;
         } //endif

        if( (rangexyz[jrec]>=rmin) & (rangexyz[jrec]<=rmax) )

       {
	  x[kpts]=rangexyz[jrec]-r0;
          kpts=kpts+1;
       }  //endif

      //...sum absolute values to compute an amplitude envelope
	  envlp[jrec]=0.0;
          for (jt=0;jt<npts;jt++)
         {
          envlp[jrec]=fabs(s1[jt])+envlp[jrec];
         } //endfor

      // estimate envelope at reference range
      if(jrec==kr0) {envlp0=envlp[jrec];}

     } //endfor
//----------------------end compute envelope-------------------


//------------apply spherical divergence correction------------
      fputs("\n\n  ENVELOPE: \n",h3);
      fputs("Trace    Measured dB   |  After Spherical Div. dB \n",h3);
      k=0;
      for(jrec=0;jrec<ntraces;jrec++)
     {
        if( (rangexyz[jrec]>=rmin) & (rangexyz[jrec]<=rmax) )
	{
	 tmp1=(double)(envlp[jrec]/envlp0);
 	 tmp2=(double)(rangexyz[jrec]/r0) * tmp1;	 
	 yc[k]=20.0*(float) (log10(tmp1));  //decibels sans sph.div.
	 y[k]=20.0*(float) (log10(tmp2)); //decibels avec sph.div.	 
		
	 fprintf(h3,"\n %3.3d  %12.0f",jrec+1,yc[k]);
         fprintf(h3,"  %12.2f",y[k]);
//         fprintf(h4," %12.2f  \n",y[k]);
	 k=k+1;
	}  //endif
     }  //endfor

//------------end spherical divergence correction--------------


//-------------least squares fit-------------------------------
      fflush(h3);
      lsqufit(x,y,kpts,0,
       &slope,&intrcpt,&lsqe,&r2);

//...plot results
      for (k=0;k<kpts;k++) {  yc[k]=slope*x[k]+intrcpt; }
#ifdef PLPLOT
      amplot(kpts,x,y,yc,slope,intrcpt,r0);
#else
      amplot(kpts,x,y,slope,intrcpt,r0);
#endif
      fputs("\n\nLEAST SQUARES FIT RESULTS:              ",h3);
      fputs("\n -----------------------------------   ",h3);
      fprintf(h3,"\n slope=%f dB/meter",slope);
      fprintf(h3,"\n intrcpt=%f dB",intrcpt);
      fprintf(h3,"\n lsqe=%f",lsqe);
      fprintf(h3,"\n Quality 0<R<1  R=%f",r2);
      neper=slope/(20.0*log10(exp((double) 1.0)));
      fprintf(h3,"\n Measured Decay nepers/m = %f",neper);

//-------------end least squares fit---------------------------
    
//...read value of dbu?
      if(argc<=4)
     {
        puts("\n      Measured Exponential Decay Effect:");
      printf("      dB/m=%f\n",slope);
      printf("  Quality 0<R<1  R=%f\n",r2);
        puts("  ENTER: exp. gain correction (neg. of effect)");
      printf("  Recommended Value: dB/m=%f\n",-slope);
      scanf("%f",&dbu);
     }  //endif

      fprintf(h3,"\n  Gain Factor Applied=%f dB/m",dbu);
      fputs("\n -----------------------------------   ",h3);
      fputs("    ",h3);
//...convert to exponential 1/m units
      neper=dbu/(20.0*log10(exp((double) 1.0)));
      fputs("\n\n  Trace  Range  Scale_Factor_Applied",h3);


//-----------------main loop---------------------------
//...display progress bar grid

      bargrid(ntraces,&bar,&ibar,&invbar);


//...trace loop

      for (jrec=0;jrec<ntraces;jrec++)
     {

//...read trace
        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        puts("ABORT--c_bsegin");
        goto quit_it;
       }

//...apply spherical divergence and exp. gain
      gainf=rangexyz[jrec]*exp((rangexyz[jrec]-r0)*neper);
      fprintf(h3,"\n  %3.3d   %7.2f   %.0f",jrec,rangexyz[jrec],gainf);

         for (jt=0;jt<npts;jt++)
        {
         s2[jt]=s1[jt]*gainf;
        }

//...output trace
        if(c_bsegout(jrec,s2,npts,&hd,h2)!=0)
       {
        puts("ABORT--c_bsegout");
        goto quit_it;
       }

//...display progress bar
      exbar(ntraces,bar,ibar,invbar,jrec+1);

     } //  end trace loop-----------------------------

      printf("\n   Listing ====>%s",lstfile);
      printf("\n   Output  ====>%s",ofile);
#ifdef PLPLOT
      printf("\n   Plot File===>bgar.ps");
#else
      printf("\n   Plot File===>bgar.pdf");
#endif
      printf("\n   Processed %d traces",ntraces);
	return(0);

quit_it:
        puts(" ");

        free(s2);
exit_s1:
        free(s1);
exit_2:
        fclose(h2);
exit_1:
        fclose(h1);
exit_3:
        fclose(h3);

quick_exit:
	return(1);

     } //  end of main

/* |--------------------------------------------------|
   | FUNCTION GETPARM                                 |
   | author: P. Michaels                              |
   | dated: 30 June 1997                              |
   |                                                  |
   | purpose: gets input parameters from command      |
   |          line or prompts user from consol.       |
   |                                                  |
   |                                                  |
   |--------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
         float *rmin, float *rmax, float *dbu, char infile[],
         char ofile[], char lstfile[], char pid[])
     {
      char              sfx1[4]="seg",
                        sfx2[4]="lst";
      char		help[3]="-h";
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"Exponential Gain Determination and/or        ");
      sprintf(msg2,"Recovery   (Sample Amplitude in Range)       ");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  %s  infile   rmin   rmax   dbu \n",pid);
      puts(" ");
      puts("     infile   =  input file name");
      puts("     rmin     =  min. range design gate  ");
      puts("     rmax     =  max. range design gate  ");
      puts("     dbu      =  gain correction to apply (dB/m) ");
      puts(" ");
      puts("NOTE: ");
      puts("       No prompt for dbu until after gain assessment. ");
      puts("     However, you may specify dbu on the command line ");
      puts("     if you already have a value you wish to use. ");
      puts(" ");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strncpy(infile,argv[1],13);
       else
      {
       puts(" Enter INPUT file name ");
       scanf(" %13s",infile);
      }
	if (namecheck(12,infile) != 0 ) return (1);

//...convert string to integer
      if(argc>2)
       *rmin=atof(argv[2]);
       else
      {
       puts(" Enter rmin (float)");
       scanf(" %f",rmin);
      }

//...convert string to float
      if(argc>3)
       *rmax=atof(argv[3]);
       else
      {
       puts(" Enter rmax (float)");
       scanf(" %f",rmax);
      }

//...convert string to float
      *dbu=0.0;
      if(argc>4)
       *dbu=atof(argv[4]);
/*     else
      {
       puts(" Enter dB correction (float)");
       scanf(" %f",dbu);
      }
*/
//...name output files
      names(infile,pid,ofile,sfx1);
      names(infile,pid,lstfile,sfx2);
       return(0);
     }  //  endif bhelp test
      } // end getparm

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(float rmin, float rmax, float dbu, char infile[],
        char pid[], char ofile[], char lstfile[],
        FILE *h3)
     {

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %s.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Input   File= %s",infile);
      fprintf(h3,"\n     Output  File= %s",ofile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fprintf(h3,"\n     Plot    File= bgar.ps ");

      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fprintf(h3,"\n     rmin= %7.2f ",rmin);
      fprintf(h3,"\n     rmax= %7.2f ",rmax);
//      fprintf(h3,"\n     dB/m= %7.2f ",dbu);
      fputs("\n",h3);                      //blank line
     }  //  end outlst
#ifdef PLPLOT
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: amplot.c                                    |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |REQUIRES PLPLOT:                                            |
   |PLPLOT is a GNU plot library which supports both C and      |
   |Fortran calls.  The distribution may be downloaded from     |
   |http://plplot.sourceforge.net                               |
   |------------------------------------------------------------|
   |                                                            |
   | x = range in meters beyond reference                       |
   | y = observed amplitude in dB                               |
   | yc = calculated dB                                         |
   |------------------------------------------------------------|
*/
/*	#include "plplot/plConfig.h"
	#include "plplot/plplot.h"
        #include <math.h>
	#include <string.h>
	#include <ctype.h>   */

      int
      amplot( PLINT npts, float x[], float y[],  float yc[],
		     float slope,  float bcept, float r0 )
     {
	PLFLT 	        fmargin,
	     		aspect,
			yjust,
			xjust;
        PLFLT		xd[maxtraces],   //double precision versions
			yd[maxtraces],   //plplot 5.3.1
			ycd[maxtraces];

	char		pltfil[]="bgar.ps";
	char		titl[80],
			xlbl[80],
			ylbl[80];
	float		xmin,xmax;
	float		ymin,ymax,
			ycmin,ycmax;
	int		kmn,kmx,k;

//  Portrait
      plscolor(0);
      plsori(1);

//...aspect ratio needed for portrait, unless you don't mind
//   plots which are squished in the vertical dimension.
//   aspect=8.5/11.0
      fmargin=0. ;
      aspect=0.77;
      yjust=0.5;
      xjust=0.0;
      plsdev("ps");

      plsdidev(fmargin,aspect,yjust,xjust);
      plsfnam(pltfil);
 
//...initialize plot
      plinit();
      plssub(1,1);
      plfontld(0);
 
//...determine maximum and minimums
      fmax_min(npts,x,&xmax,&xmin,0,&kmx,&kmn);
      fmax_min(npts,y,&ymax,&ymin,0,&kmx,&kmn);
      fmax_min(npts,yc,&ycmax,&ycmin,0,&kmx,&kmn);
      if(ycmax>ymax) ymax=ycmax;
      if(ycmin<ymin) ymin=ycmin;

 
//...set environment, window and viewport
//      plenv(xmin,xmax,ymin,ymax,0,0);
      pladv(0);
      plschr(0,.60);
      plprec(1,0);
      plssym(0,.60);
#ifdef PLPNEW
      plwidth(1.0);
#else
      plwid(1);
#endif
      plvpor(0.1, .95, 0.60, .95);
      plwind((PLFLT)xmin,(PLFLT)xmax,(PLFLT)ymin,(PLFLT)ymax);
      plbox("bcnst",0.,0.,"bcnstv",0.,0.);
//...write labels and title
      sprintf(titl,"slope=%6.4f dB/m  intercept=%6.2f dB  R0=%.3f m",
       slope,bcept,r0);
      sprintf(xlbl,"Range=(R-R0) (m) ");
      sprintf(ylbl,"Decibels ");
      pllab(xlbl,ylbl,titl);

      for (k=0;k<npts;k++) 
      {
      xd[k]=x[k];
      yd[k]=y[k];
      ycd[k]=yc[k];
      } //end for


      plpoin(npts,xd,yd,9);
 
//...plot calculated
      plline(npts,xd,ycd);
      plend();
      return(0);
     }  //end amplot
#else
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: genplot                                     |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: saves plot to a file GNUPLOT VERSION              |
   |                                                            |
   |------------------------------------------------------------|
*/
	void  amplot (int npts, float x[], float y[],
	float slope, float bcept, float r0)
	{
	FILE *pipe = popen("gnuplot -p","w");


	char	XL[]="Range=(R-R0) (m)";
	char	YL[]="Decibels";
	char	TITLE[80];

//...set title
	sprintf(TITLE,"slope=%.2f dB/m intercept= %f.2 dB  R0=%f.3 m",slope,bcept,r0);
/*
	#ifdef WIN
	char Xwin[4]="wxt";
	#else
	char Xwin[4]="x11";
	#endif
*/  
	fprintf(pipe,"set grid \n");
  //	fprintf(pipe,"set terminal %s persist \n",Xwin);
  	fprintf(pipe,"set terminal pdf color enhanced size 6.0,4.0 in \n");
	fprintf(pipe,"set output \"bgar.pdf\" \n");
  	fprintf(pipe,"set key left \n");
  	fprintf(pipe,"set xlabel \" %s \" \n",XL);
  	fprintf(pipe,"set ylabel \" %s \" \n",YL);
  	fprintf(pipe,"set title \" %s \"  \n",TITLE);
//        fprintf(pipe,"set xrange [%f:%f] \n",tmin,tmax);
  	fprintf(pipe," plot '-' u ($1):($2) w points pt 6 ps 1 lc 'black' t '' ,"); 
 	fprintf(pipe," '' u ($1):($2) w l lc 'black' lt 1 lw 2  t '' \n " );
  	int j;
  		for(j=0;j<npts;j++) {
    		fprintf(pipe,"%f %e \n",x[j],y[j]);
  		} //next j 
  	fprintf(pipe,"e\n");  //this is required when std in - plot
		for(j=0;j<npts;j++) {
		fprintf(pipe,"%e %e \n",x[j],x[j]*slope + bcept);
		} //next j
//...close pipe
	fflush(pipe);
//	getchar(); //wait for user input
	pclose(pipe);

	} //end amplot
	#endif
