// |$Id: bis2seg.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $     |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: bis2seg.c                                   |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: bis2seg  convert bison to bsegy                   |
   |          seismic processing program                        |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2016 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |  readrc     reads ascii Bison Header                       |
   |  rbison_d   reads bison data                               |
   |  rbisonhd   reads bison file header                        |
   |  s1alloc    allocate memory for input trace                |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |                                                            |
   |------------------------------------------------------------|
   |                                                            |
   |20bit Float decode in rbison_d function inspired by:        |
   |by Jens Hartmann from CWP Third Party directory             |                  
   |Institut fuer Geophysik      | phone: +49 40 4123-5046      |   
   |Universitaet Hamburg         | fax: +49 40 4123-5441        |   
   |Bundesstr. 55                | e-mail: hartmann@dkrz.d400.de|   
   |20146 Hamburg, GERMANY       |  README VERSION              |   
   |                                                            |
   |                                                            |
   |However, spikes appeared in output using Hartmann solution. |
   |Exponent decoding is a bit tricky. While the 16 bit exponent|
   |is an unsigned integer, it represents 4 exponents.  Masking |
   |each 4 bit exponent gives Hex values from 0 to F.           |
   |The problem is that the 4bit exponent should be treated as  |
   |a signed value.  To do that, I cast each 4bit value as a    |
   |singed character, then do a 2's complement conversion:      |
   |  (1) If the signed character is >7                         |
   |       Subtract 1 from the signed character.                |
   |       Complement the bits of the signed character.         |
   |    Resulting exponent -8 <= Exp <=7                        |
   | Paul Michaels, PE <pm@cgiss.boisestate.edu> 31 Oct 2015    |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/c_bsegy.h"

//...parameter definitions
	#define MAXCHAN 480
	#define NDIM 8192

//...function prototypes
      #include "../include/sub4.h"

      int  getparm(int argc,char *argv[],
        char infile[],
        char ofile[], char lstfile[], char pid[]);

      void outlst(char infile[],
        char pid[], char ofile[], char lstfile[], FILE *h3);

      int readrc(char line[], int *jrec, int *nchar, int *iexit,
        FILE *h1);

      int rbison_d(int npts, void *s1, int idsfc, FILE *h1);

      int   rbisonhd(int gidba[], int nvsa[], int *npts, int *ntraces,
	int *idsfc, int *nchan, float *fsamin, float *delayt, float *fold,
	struct header *hd, FILE *h1, FILE *h3);   	

      void *  s1alloc(int idsfc,int npts); 

      int main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h2, *h3;
      int               jt,
                        jrec,
                        npts,
                        ntraces=0,
                        ibar,
                        invbar;
      float             fsamin,
                        bar,
                        *s2;
      void		*s1;
      char              pid[5]="bis2",   //process id name
                        ofile[13],
                        lstfile[13],
                        infile[14];
      struct header     hd;

//...specific to this application
	char		buff[10],
			blnk=' ',
			scrs[5];
	int		j,
			idsfc,
			nchan;
	float		delayt,
			gaink,
			fold;
	int		gidba[50],
			nvsa[50];
//...get parameters
      if(getparm(argc,argv,
         infile,ofile,lstfile,pid)!=0)
      {
       goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...echo check to listing file
        outlst(infile,pid,ofile,lstfile,h3);

//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",infile);
          goto exit_3;
        }

//...open output file
        if( (h2=fopen(ofile,"wb"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",ofile);
          goto exit_1;
        }

//...read bison header
      if( (rbisonhd(gidba,nvsa,&npts,&ntraces,&idsfc,&nchan,&fsamin,&delayt,
	&fold,&hd,h1,h3)==0)) goto exit_2;

//...list conversions and captured values
	fprintf(h3,"\n");
	for(j=0;j<75;j++) fprintf(h3,"-");
	if (hd.tyear>90) fprintf(h3,"\n tday=%4.4d tyear=19%2.2d time=%2.2d:%2.2d",
	hd.tday,hd.tyear,hd.thour,hd.tmin);
	if (hd.tyear<90) fprintf(h3,"\n tday=%4.4d tyear=20%2.2d time=%2.2d:%2.2d",
	hd.tday,hd.tyear,hd.thour,hd.tmin);

	memcpy(&scrs[0],&hd.scrs,4);
	scrs[4]='\0';
	fprintf(h3,"\n Instrument code:  scrs=%4s",scrs);
	fprintf(h3,"\n Data format code=%d",idsfc);
	fprintf(h3,"\n Number of Channels nchan=%d",nchan);
	fprintf(h3,"\n Number of Samples npts=%d",npts);
	fprintf(h3,"\n Sample interval fsamin=%8.6f seconds",fsamin);
	fprintf(h3,"\n Delay Time delayt=%8.6f seconds",delayt);
	fprintf(h3,"\n Low Cut Filter flc=%d Hz",hd.flc);
	fprintf(h3,"\n High Cut Filter fhc=%d Hz",hd.fhc);

	ntraces=nchan;
	fprintf(h3,"\n Channel  Fold   Gain_dB ");
	for(j=0;j<nchan;j++)
	fprintf(h3,"\n    %d       %d       %d",(j+1),nvsa[j],gidba[j]);

//...assign default header values
	hd.idist=0;	//range, shot to receiver
	hd.length=(short) npts; //number of samples in trace
	hd.isi=(short) floor(fsamin*1000000.+0.5);
	hd.duse=1;

	hd.smul1=1;	//multiplier for next values
	hd.irel=0;	//receiver elevation
	hd.ishe=0;	//shot elevation
	hd.ishd=0;	//shot depth
	hd.delr=0;	//datum elevation of receiver
	hd.dels=0;	//datum elevation of source
	hd.wds=0;	//water depth of source
	hd.wdr=0;	//water depth of receiver

	hd.smul2=1;	//multiplier for next values
	hd.ishlo=0;	//shot coordinate, sx
	hd.ishla=0;	//shot coordinate, sy
	hd.irlo=0;	//receiver coordinate, rx
	hd.irla=0;	//receiver coordinate, ry

	hd.cunits=1;	//units in meters
	hd.wvel=0;	//weathering velocity
	hd.swvel=0;	//subweathering velocity
	hd.utimes=0;	//uphole time at shot
	hd.utimeg=0;	//uphole time at group
	hd.sstati=0;	//shot static (ms)
	hd.gstati=0;	//group static (ms)
	hd.tstati=0;	//total static (ms)
	hd.istime=0;	//lag time A
	hd.ibtime=0;	//lag time B
	hd.mtimes=0;	//mute start time
	hd.mtimee=0;	//mute end time
	hd.tic=1;	//trace id code
	sprintf(buff,"%c%c%c%c",blnk,blnk,blnk,blnk);
	memcpy(&hd.deploy[0],&buff[0],4);	//deployment name
	memcpy(&hd.spname[0],&buff[0],4);	//shot name
	memcpy(&hd.rstnam[0],&buff[0],4);	//receiver site name
	memcpy(&hd.shotid[0],&buff[0],4);	//shot id
	memcpy(&hd.lineid[0],&buff[0],4);	//line name
	memcpy(&hd.geoor[0],&buff[0],4);	//geophone orientation
	hd.mst=0;	//first break pick time
	hd.shtazi=0;	//source azimuth East=90
	hd.charge=0;
	hd.shtver=180;	//source polarization down
	hd.azimut=0;
	hd.geoazi=0;	//geophone azimuth East=90
	hd.geover=0;

//...dynamic allocation of seimic trace memory
       s1=s1alloc(idsfc,npts);
	if(s1==NULL)
           {
            fprintf(stderr,"ABORT:s1 can't allocate enough trace memory");
            fprintf(stderr," npts = %d",npts);
            goto exit_2;
           }  //endif

        s2=calloc(npts,sizeof(float));
          if(s2==NULL)
         {
          fprintf(stderr,"ABORT:s2 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_s1;
         }
//-----------------main loop---------------------------
//...display progress bar grid

      bargrid(ntraces,&bar,&ibar,&invbar);

//...trace loop

      for (jrec=0;jrec<ntraces;jrec++)
     {

//...read trace
        if(rbison_d(npts,s1,idsfc,h1)!=0)
       {
        puts("ABORT--rbison_d");
        goto quit_it;
       }

//...copy to output trace
	if(gidba[jrec]>=0) 
	  gaink=pow((double) 10.,(double) (gidba[jrec]/20));
	  fold=(float) nvsa[jrec];

         for (jt=0;jt<npts;jt++)
        {
	if(strcmp(scrs,"AST9")==0)
         s2[jt]=(305.176* (float) ((int *)s1)[jt]) / (gaink*fold);
	 else
	{
	if (idsfc == 1) s2[jt]=(9.54* (float) ((float *)s1)[jt]) / (gaink*fold);
        if (idsfc == 2) s2[jt]=(9.54* (float) ((int *)s1)[jt]) / (gaink*fold);
        if (idsfc == 3) s2[jt]=(9.54* (float) ((short *)s1)[jt]) / (gaink*fold);
	}
        }  // next jt

//...assign header values
	hd.nvs=nvsa[jrec];	//vertical fold
	hd.gidb=gidba[jrec];	//instrument gain in dB
	hd.tsnl=jrec+1;		//trace sequence number in line
	hd.tsnt=jrec+1;		//trace sequence number in reel
	hd.tnofr=jrec+1;	//trace number original field record

//...output trace
        if(c_bsegout(jrec,s2,npts,&hd,h2)!=0)
       {
        puts("ABORT--c_bsegout");
        goto quit_it;
       }

//...display progress bar
      exbar(ntraces,bar,ibar,invbar,jrec+1);

     } //  end trace loop-----------------------------

      printf("\n   Listing ====>%s",lstfile);
      printf("\n   Output  ====>%s",ofile);
      printf("\n   Processed %d traces",ntraces);
      return(0);

quit_it:
        puts(" ");

        free(s2);
exit_s1:
        free(s1);
exit_2:
        fclose(h2);
exit_1:
        fclose(h1);
exit_3:
        fclose(h3);

quick_exit:
	return(1);

     } //  end of main

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2016 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
         char infile[],
         char ofile[], char lstfile[],char pid[])
     {
      int               i,
			nchar;
      char              sfx1[4]="seg",
			stmp[12],
                        sfx2[4]="lst";
      char		help[3]="-h";
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"Convert Bison files (Bison Eng. Seismograph) ");
      sprintf(msg2,"to BSU (bsegy) format.  No geometry setting. ");
      hlp_labl(msg1,msg2);
      puts(" ");
      
//...help display
      puts(" ");
      printf("  bis2seg  infile \n");
      puts(" ");
      puts("     infile   =  input file name");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strncpy(infile,argv[1],13);
       else
      {
       puts(" Enter INPUT file name ");
       scanf(" %13s",infile);
      }
	if (namecheck(12,infile) != 0 ) return (1);

//...name output files
	nchar=0;
     for (i=0;i<8;i++)
	{
 	  if(infile[i]!='.') 
	  { stmp[i]=infile[i];
	    nchar=nchar+1; }
	else
	  goto label1;
	}	//next i
label1:
     stmp[nchar]=0;
     sprintf(ofile,"%s.%s",stmp,sfx1);    
     sprintf(lstfile,"%s.%s",stmp,sfx2); 
       return(0);
     }  //  endif bhelp test
      } // end getparm

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(char infile[],
        char pid[], char ofile[], char lstfile[], FILE *h3)
     {

//...print copyright label to outlst and screen

        fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: bis2seg.c ");
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Input   File= %s",infile);
      fprintf(h3,"\n     Output  File= %s",ofile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fprintf(h3,"\n     none ");
      fputs("\n",h3);                      //blank line
     }  //  end outlst
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: readrc.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: reads ascii Bison Header         	         	|
   |  jrec = byte counter for header			        |
   |  nchar= number of characters in line		        |
   |                                                            |
   |------------------------------------------------------------|
*/
      int   readrc(char line[], int *jrec, int *nchar, int *iexit,
        FILE *h1)
     {
	char		cr=13,
			lf=10,
			etx=03,
			jbyte;
	int		safety;

//...increment counter byte number in header
	safety=0;
	*nchar=0;
	while(safety<2501)
	{
	*jrec=*jrec+1;
	if(fread(&jbyte, sizeof(char), 1, h1) != 1)
	{
	fprintf(stderr,"Error reading Bison Header");
	return(1);
	}	//endif
//...character count for line array
	*nchar=*nchar+1;
	
//...test for double etx characters
	if(jbyte == etx )
	{
		*jrec=*jrec+1;
		if(fread(&jbyte, sizeof(char), 1, h1) != 1)
		{
		fprintf(stderr,"Error reading Bison Header");
		return(1);
		}	//endif
	}	//endif
	if(jbyte == etx )
	{ *iexit=1;
	  goto label6;
	}	//endif
//...test for end of line
	if(jbyte == cr)
	{
		*jrec=*jrec+1;
		if(fread(&jbyte, sizeof(char), 1, h1) != 1)
		{
		fprintf(stderr,"Error reading Bison Header");
		return(1);
		}	//endif
		  if(jbyte == lf)
		  {
		  *nchar=*nchar-1;
		  goto label6;
		  }  //endif
	}
	  else
	{
	line[*nchar-1]=jbyte;
	}	//endif
	safety=safety+1;
	}	//endwhile

//...safety check
	if(safety>=2500)
	{ fprintf(stderr,"ABORT--etx character not found");
	fprintf(stderr," May not be BISON FILE");
	return(1);
	}	//endif

label6:
	line[*nchar]=0;
	return(0);
     }  //end readrc

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: rbison_d.c                                  |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: reads bison data                 	         	|
   |                                                            |
   |------------------------------------------------------------|
*/
      int   rbison_d(int npts, void *s1, int idsfc, FILE *h1) 
     {
	short   	f_sample[4];
	unsigned short 	gain;
	signed char     GainL[4];
	long		j;
	int		k;
	if(idsfc < 1 || idsfc >3)
	{	fprintf(stderr,"\nABORT--Unrecognized data format");
		fprintf(stderr,"\n Not Float or Integer");
		return(1);
	}	//endif
	switch(idsfc)
	{
		case 1:
		{	
/* Float 16 bit 2's complement mantissa, 4 bit exponent normalized float
   generated from 32 bit internal fixed format standard instrument storage
   format. Transmitted as follows: in groups of 5 words

   LSbyte of first sample mantissa
   MSbyte of first sample mantissa
   LSbyte of second sample mantissa
   MSbyte of second sample mantissa
   LSbyte of third sample mantissa
   MSbyte of third sample mantissa
   LSbyte of fourth sample mantissa
   MSbyte of fourth sample mantissa

   Exponent word:
     Least significant nibble of first byte for fourth word.
     Most significant nibble of first byte for third word.
     Least significant nibble of second byte for second word.
     Most significant nibble of first byte for first word. */

	  for (j=0;j<npts;j+=4)
	    {
	      fread(&f_sample,sizeof(short),4,h1);
	      fread(&gain,sizeof(unsigned short),1,h1);
		GainL[0]=(signed char) ((gain & 0xf000)>>12);
		GainL[1]=(signed char) ((gain & 0x0f00)>>8);
		GainL[2]=(signed char) ((gain & 0x00f0)>>4);
		GainL[3]= (signed char) ((gain & 0x000f)>>0);

/* Check to see if 4 bit exponent is greater than 7 (implies negative 
exponent in that case).  Decode to negative 4 bit number using
assumption of 2's complement  P. Michaels 31 Oct 2015
*/
	for (k=0; k<4; k++) {
		if (GainL[k] > 7 ) {
		  GainL[k]=GainL[k]-1;
		  GainL[k]=~GainL[k];
		  }
	} //next k

	       ((float *)s1)[j]=(float)( f_sample[0]*pow((double) 2,((double) GainL[0] )));
	       ((float *)s1)[j+1]=(float)( f_sample[1]*pow((double) 2,((double) GainL[1])));
	       ((float *)s1)[j+2]=(float)( f_sample[2]*pow((double) 2,((double) GainL[2])));
	       ((float *)s1)[j+3]=(float)( f_sample[3]*pow((double) 2,((double) GainL[3])));
/*
// Debug section
if (j==56) {
fprintf(stdout,"f_sample[0:3]= %d %d %d %d \n",f_sample[0],f_sample[1],f_sample[2],f_sample[3]);
fprintf(stdout,"gain= %4.4x \n", gain);
fprintf(stdout," G>>12 =%4.4x sample=%d \n", (gain & 0xf000)>>12, j+1);
fprintf(stdout," G>>8  =%4.4x sample=%d \n", (gain & 0x0f00)>>8, j+2);
fprintf(stdout," G>>4  =%4.4x sample=%d \n", (gain & 0x00f0)>>4, j+3);
fprintf(stdout," G>>0  =%4.4x sample=%d \n", (gain & 0x000f)>>0, j+4);
fprintf(stdout," %4.4x = %4.4d  sample=%d \n", GainL[0], GainL[0], j+1);
fprintf(stdout," %4.4x = %4.4d  sample=%d \n", GainL[1], GainL[1], j+2);
fprintf(stdout," %4.4x = %4.4d  sample=%d \n", GainL[2], GainL[2], j+3);
fprintf(stdout," %4.4x = %4.4d  sample=%d \n", GainL[3], GainL[3], j+4);
exit(1);
} //endif
*/
            }  //next j


		return(0);
		break;
		}	//esac 1
		case 2:
		{
		if(fread((int *)s1, sizeof(int), npts, h1) != npts)
		{
		fprintf(stderr,"Error reading Bison Data (4 byte integer)");
		return(1);
		}	//endif
		else
		return(0);
		break;
		}	//esac 2
		case 3:
		{
		if(fread((short *)s1, sizeof(short), npts, h1) != npts)
		{
		fprintf(stderr,"Error reading Bison Data (2 byte integer)");
		return(1);
		}	//endif
		else
		return(0);
		break;
		}	//esac 3
	}	//endswitch
		return(1);
     }  //end rbison_d	

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: rbisonhd.c                                  |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: reads bison file header          	         	|
   |                                                            |
   |------------------------------------------------------------|
*/
      int   rbisonhd(int gidba[], int nvsa[], int *npts, int *ntraces,
	int *idsfc, int *nchan, float *fsamin, float *delayt, float *fold,
	struct header *hd, FILE *h1, FILE *h3)   	
      {
	char		line[81],
			buff[10];
	int		ntr,
			jrec,
			kk,
			jchac,
			nchar,
			iexit;

	iexit=0;
	ntr=0;
	jchac=0;
	while(iexit == 0)
	{
          if(readrc(line,&jrec,&nchar,&iexit,h1) !=0 )
         {
          return(0) ;
         }
	ntr=ntr+1;
	if(ntr < 26) fprintf(h3,"\n %4.4d %s",ntr,line);

//...capture data sample format code
	if(ntr==3)
	{
	memcpy(&buff,&line[0],4);
	(*hd).tday= atoi(buff);
	memcpy(&buff,&line[4],2);
	buff[2]='\0';
	(*hd).tyear= atoi(buff);
	memcpy(&buff,&line[6],2);
	buff[2]='\0';
	(*hd).thour= atoi(buff);
	memcpy(&buff,&line[8],2);
	buff[2]='\0';
	(*hd).tmin= atoi(buff);
	}	//endif
//...capture instrument type
	if(ntr==5)
	{
	sscanf(&line[0],"%4s",&buff[0]);
	memcpy(&(*hd).scrs,&buff[0],4);
	buff[4]='\0';
	}	//endif
//...data format code
	if(ntr==8)
	{
	memcpy(&buff,&line[1],1);
	if(buff[0]=='W') *idsfc=3;
	if(buff[0]=='L') *idsfc=2;
	if(buff[0]=='F') *idsfc=1;
	}	//endif
//...number of channels
	if(ntr==15)
	{
	memcpy(&buff,&line[0],4);
	buff[4]='\0';
	*nchan = atoi(buff);
	  if(*nchan>MAXCHAN)
	  { fprintf(stderr,"\n ABORT nchan=%d",*nchan);
	    return(0);
	  }  //endif 
	}	//endif
//...number of samples 
	if(ntr==17)
	{
	memcpy(&buff,&line[0],6);
	buff[6]='\0';
	*npts = atoi(buff);
	  if(*npts>NDIM)
	  { fprintf(stderr,"\n ABORT npts=%d",*npts);
	    return(0);
	  }  //endif 
	}	//endif
//...sample interval 
	if(ntr==18)
	{
	memcpy(&buff,&line[0],4);
	buff[4]='\0';
	*fsamin = atof(buff)/1000. ;
	}	//endif
//...delay time 
	if(ntr==19)
	{
	memcpy(&buff,&line[0],4);
	buff[4]='\0';
	*delayt = atof(buff)/1000. ;
	(*hd).ictime = floor(1000*(*delayt));
	}	//endif
//...low cut filter 
	if(ntr==20)
	{
	memcpy(&buff,&line[0],4);
	buff[4]='\0';
	(*hd).flc = atoi(buff) ;
	}	//endif
//...high cut filter 
	if(ntr==21)
	{
	memcpy(&buff,&line[0],4);
	buff[4]='\0';
	(*hd).fhc = atoi(buff) ;
	}	//endif
//...channel information on gain and fold
	if(ntr>25)
	{
	if((ntr-25)%8==1) jchac=jchac+1;
	kk=(jchac-1)*8+26;
	  if(ntr==kk)
	  {
           memcpy(&buff,&line[0],4);
	   buff[4]='\0';
	   nvsa[jchac-1] = atoi(buff) ;
	  }	//endif
	  if(ntr==(kk+1))
	  {
           memcpy(&buff,&line[0],4);
	   buff[4]='\0';
	   gidba[jchac-1] = atoi(buff) ;
	  }	//endif
	}	//endif
	}  // endwhile
	return(1);
     }  //end rbisonhd

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: s1alloc.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: allocate memory for input trace	         	|
   |                                                            |
   |------------------------------------------------------------|
*/
      void *s1alloc(int idsfc, int npts) 
     {
	switch(idsfc)
	{
	  case 1:
	  {
          return( (float *) calloc(npts,sizeof(float)) );
	  break;
	  }  //esac 1
	  case 2:
	  {
          return( (int *) calloc(npts,sizeof(int)) );
	  break;
	  }  //esac 2
	  case 3:
	  {
          return( (short *) calloc(npts,sizeof(short)) );
	  break;
	  }  //esac 3
	  default:
	   {
	    fprintf(stderr,"\n ABORT--Unrecognized format type");
	    return(NULL);
	   }
	}	//end switch
      }	//end s1alloc
