// |$Id: bplt.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $              |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: bplt.c                                      |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: bplt plot seismic traces                          |
   |                                                            |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |REQUIRES PLPLOT:                                            |
   |PLPLOT is a GNU plot library which supports both C and      |
   |Fortran calls.  The distribution may be downloaded from     |
   |http://plplot.sourceforge.net                               |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |  tplot      plot traces                                    |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <ctype.h>
      #include <time.h>
      #include "../include/c_bsegy.h"
#ifdef PLPLOT
      #include "plConfig.h"
      #include "plplot.h"
#endif

//...parameter definitions
      #define  maxtraces 500

//...function prototypes
      #include "../include/sub4.h"

      int  getparm(int argc,char *argv[],
         int *idev, int *iorient, int *itype, 
	 int *iftr, int *iltr, float *ftmin, float *ftmax,
         int *istyl, float *amp, float *percnt, 
	 float *xaxis, float *yaxis, char infile[],
         char ofile[], char lstfile[], char pid[]);

      void outlst(
        int idev, int iorient, int itype, int iftr, int iltr, 
       	float ftmin, float ftmax, int istyl, float amp, float percnt,
        char infile[], char pid[], char pltfil[], char lstfile[],
        FILE *h3);

#ifdef PLPLOT
//PLPLOT Version
      int tplot( PLINT npts, PLFLT x[], PLFLT s[], int jrec, float vzero,  
	  int iflip, float maxx, float minx, float smax, float smin,
	  int idev, int iorient, int itype, char infile[], int istyl, 
	  float amp, float percnt, float xaxis, float yaxis, int *irun);

     void pm_plfill(PLFLT x[], PLFLT s[], PLINT kpts, PLINT j, PLFLT vzerod,
	  PLINT fstyl);
#else
//Gnuplot Version
//portrait version (y,tm)
      void genplotA (float tmin, float tmax, char infile[], FILE *pipe,
	 int itype, float minx, float maxx, float dx, int idev, char pltfil[],
	 float amp, float xaxis, float yaxis );
      void genplotB (int ntraces, FILE *pipe, int istyl, float x[] );
//landscape version (tm,y)
      void genplotAL (float tmin, float tmax, char infile[], FILE *pipe,
	 int itype, float minx, float maxx, float dx, int idev, char pltfil[],
	 float amp, float xaxis, float yaxis );
      void genplotBL (int ntraces, FILE *pipe, int istyl, float x[] );

      void genplot (float tm[], float y[], int itrace, int npts, FILE *pipe );

#endif

      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h3 ;
      int               jt,
                        jrec,
                        npts,
                        ntraces,
                        ibar,
                        invbar;
      float             fsamin,
                        bar,
                        *s1,
                        *s2;
#ifdef PLPLOT
      PLFLT		*pls1, 		 //these are defined in plplot
			*pls2;      	 //adapts to single or double prec.
#else
      float		*pls1,
			*pls2;
#endif
      char              pid[5]="bplt",   //process id name
                        ofile[13],
			pltfil[20],
                        lstfile[13],
                        infile[14];
      struct header     hd;

//...specific to this application
      float             amp;            //amplitude for 1 trace deflection
      float             sx, sy, sz, sd;  //source coordinates
      float             rx, ry, rz;      //receiver coordinates
      float             range;           //source-receiver separation
                                         //stored geometry info
      float             x[maxtraces];    //non-time array
      float		percnt;		 //overplot percent (100=1 trace)
      float		ftmin,ftmax;	 //min and max time to plot
      int		iftr,iltr;	 //first and last trace to plot

      float             maxx,minx,       //actual max,min range in non-time 
                        maxs,mins,       //max and minimum trace absolute value
                        smax,smin;       //max and minimum for entire data set 
      float  		dx,		 //trace interval in chosen units
      			vzero;		 //biased zero point on a trace in chosen units

      int		kmx,kmn;	 //indexes for max and min values
      int		idev,		 //plot device
      			iorient,	 //plot orientation
			itype;		 //non-time axis type
      int		jmin,jmax;	 //time gate			
      int		k,kpts;		 //time aperture variables
      float		tmx;		 //max time variable
#ifdef PLPLOT
      int		irun;		 //switch to initialize plot
#endif
      int		istyl;		 //plot style switch
      int		iflip;		 //flip plot direction switch
      float		xaxis,yaxis;	 //axis lengths in inches

//...get parameters
      if(getparm(argc,argv,&idev,&iorient,&itype,&iftr,&iltr,
	&ftmin,&ftmax,&istyl,&amp,&percnt,&xaxis,&yaxis,
        infile,ofile,lstfile,pid)!=0)
      {
       goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",infile);
          goto exit_3;
        }

//...check input file size
          if(in_chk(&ntraces,&npts,&fsamin,hd,h1)!=0)
         {
          goto exit_2;
         }
//...apply limits on aperture to plot
	if (ftmin<0) ftmin=0.;
	tmx=(float)(npts-1)*fsamin;
	if (ftmax>tmx) ftmax=tmx;
	if (iftr<1) iftr=1;
	if (iltr>ntraces) iltr=ntraces;
	iftr=iftr-1;
	iltr=iltr-1;  //c uses subscripts from 0 not 1
	jmin=ftmin/fsamin;
	jmax=ftmax/fsamin;
	if (jmin<0) jmin=0;
	if (jmax>npts) jmax=npts-1;
	if (jmin>npts) jmin=npts-2;
	if (jmax<jmin) jmax=jmin+1;

      switch (idev)
      {
	      case 0:
      {
      sprintf(pltfil,"none X");
      break;}
       	      case 1:
      {
      sprintf(pltfil,"bplt.ps");
      break;}
       	      case 2:
      {
      sprintf(pltfil,"bplt.fig");
      break;}
       	      case 3:
      {
      sprintf(pltfil,"bplt.jpg");
      break;}
       	      case 4:
      {
      sprintf(pltfil,"bplt.pdf");
      break;}
      } //endswitch

//...echo check to listing file
      outlst(idev,iorient,itype,iftr,iltr,ftmin,ftmax,
	     istyl,amp,percnt,infile,pid,pltfil,lstfile,h3);

	if (idev == 0) {
#ifdef PLPLOT
	fprintf(stdout,"\n . . .press enter to end plot \n");
#else
	if (idev == 0) {
	fprintf(stdout,"\n . . .Zoom using right click-drag \n");
	fprintf(stdout,"\n . . .press any key to exit zoom\n");
	fprintf(stdout,"\n . . .press q to end plot \n");
	} //endif
#endif	
	} //endif

//...print input file data to lstfile
        fprintf(h3,"\n     Number of samples= %d",npts);
        fprintf(h3,"\n     Sample Interval = %5.4f sec.",fsamin);
        fprintf(h3,"\n     Number of traces = %d",ntraces);

//...check number traces
        if(ntraces>maxtraces)
        {
         puts(" ABORT  !!! number of traces exceeds dimension");
         goto exit_2;
        }

//...dynamic allocation of seimic trace memory
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          puts("ABORT:s1 can't allocate enough trace memory");
          printf(" npts = %d",npts);
          goto exit_4;
         }
        s2=calloc(npts,sizeof(float));
          if(s2==NULL)
         {
          puts("ABORT:s2 can't allocate enough trace memory");
          printf(" npts = %d",npts);
          goto exit_s1;
         }
#ifdef PLPLOT
//...dynamic allocation of seimic trace memory
        pls1=calloc(npts,sizeof(PLFLT));
          if(pls1==NULL)
         {
          puts("ABORT:pls1 can't allocate enough trace memory");
          printf(" npts = %d",npts);
          goto exit_4;
         }
        pls2=calloc(npts,sizeof(PLFLT));
          if(pls2==NULL)
         {
          puts("ABORT:pls2 can't allocate enough trace memory");
          printf(" npts = %d",npts);
          goto exit_s1;
         }
#else
        pls1=calloc(npts,sizeof(float));
          if(pls1==NULL)
         {
          puts("ABORT:pls1 can't allocate enough trace memory");
          printf(" npts = %d",npts);
          goto exit_4;
         }
        pls2=calloc(npts,sizeof(float));
          if(pls2==NULL)
         {
          puts("ABORT:pls2 can't allocate enough trace memory");
          printf(" npts = %d",npts);
          goto exit_s1;
         }
#endif

//---------------------offset range check, list----------------
//...header for geometry listing
      fputs("\n\n Geometry Information ",h3);
      fputs("\n trace  offset    rx      ry      rz ",h3);
      fputs("    sx       sy      sz\n",h3);

// reset max and minimum registers for data set
      smax= -1.0E-30;
      smin= 1.0E+30;

      for (jrec=0;jrec<ntraces;jrec++)
     {
//...read trace
        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        puts("ABORT--c_bsegin");
        goto quit_it;
       }

//...find max and minimum absolute values in trace
      fmax_min(npts,s1,&maxs,&mins,1,&kmx,&kmn);
      if (maxs>smax) {smax=maxs;}
      if (mins<smin) {smin=mins;}

//...determine shot and receiver coordinates
      findxyz(&sx,&sy,&sz,&sd,&rx,&ry,&rz,&range,hd);

//...print coordinates to listing
      fprintf(h3,"  %3.3d %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f \n",
        jrec+1,range,rx,ry,rz,sx,sy,sz);
fflush(h3); //debug
//...store geometry data in arrays
      switch(itype)
      {
	      case 0:
	      {
		      x[jrec]=(float) jrec+1;
	       break;
	      } // 0 esac
	      case 1:
	      {
		      x[jrec]=range;
	       break;
	      } // 0 esac
	      case 2:
	      {
		      x[jrec]=rz;
	       break;
	      } // 0 esac
	      case 3:
	      {
		      x[jrec]=rx;
	       break;
	      } // 0 esac
	      case 4:
	      {
		      x[jrec]=ry;
	       break;
	      } // 0 esac
	      case 5:
	      {
		      x[jrec]=sz;
	       break;
	      } // 0 esac
	      case 6:
	      {
		      x[jrec]=sx;
	       break;
	      } // 0 esac
	      case 7:
	      {
		      x[jrec]=sy;
	       break;
	      } // 0 esac
      } //end switch	      
     } //endfor

//...find max and minimum range
      fmax_min(ntraces,x,&maxx,&minx,0,&kmx,&kmn);

      fprintf(h3,"\n Absolute Trace Value=%f\n",smax);
      fprintf(h3," Min Absolute Trace Value=%f\n",smin);
      fprintf(h3," Max  Non-Time Value=%f\n",maxx);
      fprintf(h3," Min  Non-Time Value=%f\n",minx);

// define dx, trace interval in chosen units
      if (ntraces ==1 )
	{dx=1; }
	else
	{
	dx=(maxx-minx)/(float) (ntraces - 1);
	}
      fprintf(h3," DX=%f\n",dx);
fflush(h3); //debug
//set percent overplot to fraction
      percnt=percnt/100.;

//-----------------main loop---------------------------
	 //redefine smin and smax for plot
	 smax=((float)(jmin))*fsamin*1000.;  //units are ms
	 smin=((float)(jmax))*fsamin*1000.;

//...display progress bar grid

      bargrid(ntraces,&bar,&ibar,&invbar);

//...apply space aperture
         minx=x[iftr]; //first trace plotted first (left)
         maxx=x[iltr]; //last trace plotted last (right)     
	 iflip=0;
	 if(minx>maxx) iflip=1;

//...trace loop
#ifdef PLPLOT
      irun=1;
#endif

#ifndef PLPLOT
	FILE *pipe = popen("gnuplot ","w");
	FILE *h5 = fopen("bplt.gp","w");
	ntraces=iltr-iftr+1;	
	if (iorient == 1) {
	genplotA (smin,smax,infile,pipe,itype,minx,maxx,dx,
		 idev,pltfil,amp,xaxis,yaxis);
	genplotA (smin,smax,infile,h5,itype,minx,maxx,dx, 
		 idev,pltfil,amp,xaxis,yaxis);
	genplotB (ntraces,pipe,istyl,&x[iftr]);
	genplotB (ntraces,h5,istyl,&x[iftr]);
	} else {
	genplotAL (smin,smax,infile,pipe,itype,minx,maxx,dx,
		 idev,pltfil,amp,xaxis,yaxis);
	genplotAL (smin,smax,infile,h5,itype,minx,maxx,dx,
		 idev,pltfil,amp,xaxis,yaxis);
	genplotBL (ntraces,pipe,istyl,&x[iftr]);
	genplotBL (ntraces,h5,istyl,&x[iftr]);
	}
#endif

      for (jrec=iftr;jrec<iltr+1;jrec++)
     {

//...read trace
        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        puts("ABORT--c_bsegin");
        goto quit_it;
       }

         vzero=x[jrec]; //biased zero for trace

	 switch (iflip)
	 {
	 case 0:
	{
           for (jt=0;jt<npts;jt++)
          {
           s1[jt]=vzero+dx*(s1[jt]/(amp)) ; //bias signal
	   if (s1[jt]-vzero > (dx/2)*percnt) //apply clip
	     {s1[jt]=vzero+(dx/2)*percnt;}
	   if (s1[jt]-vzero < (-dx/2)*percnt)
	   {s1[jt]=vzero-(dx/2)*percnt;}

	   s2[jt]=((float) jt) * fsamin * 1000.;
        } //next jt
	 break;
	} // esac 0
	
	case 1:
	{
         for (jt=0;jt<npts;jt++)
        {
         s1[jt]=vzero-dx*(s1[jt]/(amp)) ; //bias signal,flip
	 if (s1[jt]-vzero > (dx/2)*percnt) //apply clip
	   {s1[jt]=vzero+(dx/2)*percnt;}
	 if (s1[jt]-vzero < (-dx/2)*percnt)
	 {s1[jt]=vzero-(dx/2)*percnt;}

	 s2[jt]=((float) jt) * fsamin * 1000.;
        }
	 break;
	} // esac 1
	 } //endswitch iflip

//...apply time aperture
	 k=0;
	 kpts=jmax-jmin+1;
#ifdef PLPLOT
      for (jt=jmin;jt<jmax+1;jt++)
	{
	 pls1[k]=(PLFLT) s1[jt];
	 pls2[k]=(PLFLT) s2[jt];
	 k=k+1;
	} //next jt

      tplot(kpts,pls1,pls2,jrec,vzero,iflip,
        maxx,minx,smax,smin,idev,iorient,
	itype,infile,istyl,amp,percnt,xaxis,yaxis,&irun);  
#else
      for (jt=jmin;jt<jmax+1;jt++)
	{
	 pls1[k]= s1[jt];
	 pls2[k]= s2[jt];
	 k=k+1;
	} //next jt

//...plot traces
	genplot (pls2,pls1,jrec,kpts,pipe);
	genplot (pls2,pls1,jrec,kpts,h5);
#endif

//...display progress bar
      exbar(ntraces,bar,ibar,invbar,jrec+1);

     } //  end trace loop-----------------------------
#ifndef PLPLOT
//...close pipe
  	fflush(pipe);
#ifndef WIN
	if (idev == 0) {
  	getchar();} //wait for user input
#endif  	
pclose(pipe);
#endif



#ifdef PLPLOT
      //close the plot
      plend();
#endif
      
      printf("\n   Listing ====>%s",lstfile);
      if (idev==1)
      printf("\n   PostScript Plot File===>bplt.ps");
      if (idev==2)
      printf("\n   XFIG Plot File===>bplt.fig");
      if (idev==3)
      printf("\n   JPEG Plot File===>bplt.jpg");
      if (idev==4)
      printf("\n   PDF  Plot File===>bplt.pdf");
      printf("\n   Processed %d traces",ntraces);
	return(0);

quit_it:
        puts(" ");
	free(pls2);
        free(pls1); 
        free(s2);
exit_s1:
        free(s1);
exit_4:
exit_2:
        fclose(h1);
exit_3:
        fclose(h3);

quick_exit:
	return(1);

     } //  end of main

/* |--------------------------------------------------|
   | FUNCTION GETPARM                                 |
   | author: P. Michaels                              |
   | dated: 06 May 2003                               |
   |                                                  |
   | purpose: gets input parameters from command      |
   |          line or prompts user from consol.       |
   |                                                  |
   |                                                  |
   |--------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
         int *idev, int *iorient, int *itype, 
	 int *iftr, int *iltr, float *ftmin, float *ftmax,
         int *istyl, float *amp, float *percnt,
	 float *xaxis, float *yaxis, char infile[],
         char ofile[], char lstfile[], char pid[])
     {
      char              sfx1[4]="seg",
                        sfx2[4]="lst";
      char		help[3]="-h";
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"Plot Seismic Traces using X or PostScript    ");
      sprintf(msg2,"for the output device                        ");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  %s  infile idev iorient itype 1tr Ltr tmin tmax istyl"
		     " amp percnt xaxis yaxis \n",
	   pid);
      puts(" ");
      puts("     infile   =  input file name");
      puts("     idev     =  output device  ");
      puts("             0=    xwin/wxt  (Linux/MS Windows) ");
      puts("             1=    Post Script               ");
      puts("             2=    xfig                      ");
      puts("             3=    jpeg                      ");
      puts("             4=    PDF                       ");
      puts("     iorient  =  orientation  ");
      puts("             0=    landscape ");
      puts("             1=    portrait ");
      puts("     itype    =  select non-time axis type       ");
      puts("             0=    trace number");
      puts("             1=    offset      ");
      puts("             2=    geophone z-coord  ");
      puts("             3=    geophone x-coord  ");
      puts("             4=    geophone y-coord  ");
      puts("             5=    shot z-coord      ");
      puts("             6=    shot x-coord      ");
      puts("             7=    shot y-coord      ");
      puts("     1tr      =  first trace to plot ");
      puts("     Ltr      =  last trace to plot ");
      puts("     tmin     =  minimum time to plot ");
      puts("     tmax     =  maximum time to plot");
      puts("     istyl    =  style of plot");
      puts("             0=    wiggle plot  ");
      puts("             1=   black/white variable area");
      puts("             2=   black/grey variable area");
      puts("     amp      =  amplitude for 1 trace deflection");
      puts("     percnt   =  percent overplot 100= 1 trace  ");
      puts("     xaxis    =  length of x-axis (non-time) in inches");
      puts("     yaxis    =  length of y-axis (time) in inches");
      puts("     (if xaxis and yaxis absent, 6.0 by 4.0 inches     ");
      puts(" ");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strncpy(infile,argv[1],13);
       else
      {
       puts(" Enter INPUT file name ");
       scanf(" %13s",infile);
      }
	if (namecheck(12,infile) != 0 ) return (1);

//...convert string to integer
      if(argc>2)
       *idev=atoi(argv[2]);
       else
      {
       puts(" Enter idev  (int) ");
       puts("  0=xwin ");
       puts("  1=PostScript ");
       puts("  2=xfig  ");
       puts("  3=jpeg  ");
       puts("  4=PDF   ");
       scanf(" %d",idev);
      }

//...convert string to int  
      if(argc>3)
       *iorient=atoi(argv[3]);
       else
      {
       puts(" Enter iorient (int)");
       puts("  0=landscape ");
       puts("  1=portrait ");
       scanf(" %d",iorient);
      }

//...convert string to int  
      if(argc>4)
       *itype=atoi(argv[4]);
       else
      {
       puts(" Enter itype (int)");
       puts("             0=  trace number");
       puts("             1=  offset      ");
       puts("             2=  geophone z-coord  ");
       puts("             3=  geophone x-coord  ");
       puts("             4=  geophone y-coord  ");
       puts("             5=  shot z-coord      ");
       puts("             6=  shot x-coord      ");
       puts("             7=  shot y-coord      ");
       scanf(" %d",itype);
      }

//...convert string to int  
      if(argc>5)
       *iftr=atoi(argv[5]);
       else
      {
       puts(" Enter 1tr  (int) ");
       puts("     1tr      =  first trace to plot");
       scanf(" %d",iftr);
      }
//...convert string to int  
      if(argc>6)
       *iltr=atoi(argv[6]);
       else
      {
       puts(" Enter Ltr (int)    ");
       puts("     Ltr      =  last trace to plot ");
       scanf(" %d",iltr);
      }
//...convert string to float
      if(argc>7)
       *ftmin=atof(argv[7]);
       else
      {
       puts(" Enter tmin (float) ");
       puts("     tmin     =  minimum time to plot ");
       scanf(" %f",ftmin);
      }
//...convert string to float
      if(argc>8)
       *ftmax=atof(argv[8]);
       else
      {
       puts(" Enter tmax (float) ");
       puts("     tmax     =  maximum time to plot");
       scanf(" %f",ftmax);
      }

//...convert string to int  
      if(argc>9)
       *istyl=atoi(argv[9]);
       else
      {
       puts(" Enter istyl (int)  ");
      puts("     0=  wiggle plot  ");
      puts("     1=   black/white variable area");
      puts("     2=   black/grey variable area");
       scanf(" %d",istyl);
      }

//...convert string to float
      if(argc>10)
       *amp=atof(argv[10]);
       else
      {
       puts(" Enter amp (float) ");
       puts("     amp     =  amplitude for 1 trace deflection");
       scanf(" %f",amp);
      }

//...convert string to float
      if(argc>11)
       *percnt=atof(argv[11]);
       else
      {
       puts(" Enter percnt (float) ");
       puts("     percnt     =  percent overplot 100 = 1 trace");
       scanf(" %f",percnt);
      }
  
//...convert string to float
      if(argc>12)
       *xaxis=atof(argv[12]);
       else
      {
	      *xaxis=-1.0;
      }
  
//...convert string to float
      if(argc>13)
       *yaxis=atof(argv[13]);
       else
      {
	      *yaxis=-1.0;
      }
  
//...name output files
      names(infile,pid,ofile,sfx1);
      names(infile,pid,lstfile,sfx2);
       return(0);
     }  //  endif bhelp test
      } // end getparm

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(
        int idev, int iorient, int itype, int iftr, int iltr, 
       	float ftmin, float ftmax, int istyl, float amp, float percnt,
        char infile[], char pid[], char pltfil[], char lstfile[],
        FILE *h3)
     {

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %s.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Input   File= %s",infile);
//      fprintf(h3,"\n     Output  File= %s",ofile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fprintf(h3,"\n     Plot    File= %s",pltfil);

      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fprintf(h3,"\n  idev   = %d ",idev);
      fprintf(h3,"\n  iorient= %d ",iorient);
      fprintf(h3,"\n  itype  = %d ",itype);
      fprintf(h3,"\n  1tr    = %d ",iftr );
      fprintf(h3,"\n  Ltr    = %d ",iltr );
      fprintf(h3,"\n  tmin   = %f ",ftmin);
      fprintf(h3,"\n  tmax   = %f ",ftmax);
      fprintf(h3,"\n  istyl  = %d ",istyl);
      fprintf(h3,"\n  amp   = %7.2f ",amp);
      fprintf(h3,"\n  percnt = %7.2f ",percnt);
      fputs("\n",h3);                      //blank line
     }  //  end outlst
#ifdef PLPLOT
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: tplot.c                                     |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |REQUIRES PLPLOT:                                            |
   |PLPLOT is a GNU plot library which supports both C and      |
   |Fortran calls.  The distribution may be downloaded from     |
   |http://plplot.sourceforge.net                               |
   |------------------------------------------------------------|
   | x=sample values biased to trace location and new units     |
   | s=sample time                                              |
   |                                                            |
   |------------------------------------------------------------|
*/
/*      following includes already in main (comment out here)   */
/*	#include "plplot/plConfig.h"
	#include "plplot/plplot.h"  
        #include <math.h>
	#include <string.h>
	#include <ctype.h>   */

      int tplot( PLINT npts, PLFLT x[], PLFLT s[], int jrec, float vzero,  
	  int iflip, float maxx, float minx, float smax, float smin,
	  int idev, int iorient, int itype, char infile[], int istyl, 
	  float amp, float percnt, float xaxis, float yaxis, int *irun)
     {
	PLFLT		fmargin,
	     		aspect,
			yjust,
			xjust;
	PLFLT		svx0,
			svx1,
			svy0,
			svy1;
/*	PLFLT		gxmin,gxmax,
			gymin,gymax;  */
/*	PLINT		mark=100,
			space=400; */
	PLFLT		slope0,slope1;
	PLFLT		fsgn,vzerod;
	PLINT		j,k,kpts;
	PLINT		inc[2],del[2];
	char		pltfil[]="bplt.jpg ";
	char		titl[180],
			titl2[180],
			xlbl[180],
			ylbl[180];
//set default size
	if (xaxis < 0.0 ) xaxis=6.0;
	if (yaxis < 0.0 ) yaxis=4.0;

// copy to PLFLT 
	vzerod=(PLFLT)vzero;
// First trace setup
	if (*irun==1)
	{
//  reset first run
      *irun=0;
      plscolor(0);
      plsori(1);


// if trace number plot
      if (itype==0)
      { maxx=maxx+1;
	minx=minx-1; }
      else  // 5% margin
      { svx0=(maxx-minx);
        maxx=svx0*.05+maxx;
	minx=minx-svx0*.05;}

//...aspect ratio needed for portrait, unless you don't mind
//   plots which are squished in the vertical dimension.
//   aspect=8.5/11.0
      fmargin=0. ;
      if(iorient==1)
        aspect=0.77;
       else
        aspect=1./.77;	
      yjust=0.5;
      xjust=0.0;
      switch (idev)
      {
	      case 0:
      {
      plssub(1,1);
      plsdiori(iorient);
      xaxis=-1.0;
      plsdev("xwin");
      plscolbg(255,255,255);
#ifdef PLPNEW
      plwidth(4.0);
#else
      plwid(4);
#endif
      break;}
       	      case 1:
      {
      plssub(1,1);
      plsdiori(iorient);
      sprintf(pltfil,"bplt.ps");
      plsdev("ps");
      plsdidev(fmargin,aspect,yjust,xjust);
      plsfnam(pltfil);   
#ifdef PLPNEW
      plwidth(2.0);
#else
      plwid(2);
#endif
      break;}
       	      case 2:
      {
      plssub(1,1);
      plsdiori(iorient);
      sprintf(pltfil,"bplt.fig");
      plsdev("xfig");
      plscolbg(255,255,255);
      plsdidev(fmargin,aspect,yjust,xjust);
      plsfnam(pltfil);   
#ifdef PLPNEW
      plwidth(1.0);
#else
      plwid(1);
#endif
      break;}
       	      case 3:
      {
      plssub(1,1);
      plsdiori(iorient);
      xaxis=-1.0;
      sprintf(pltfil,"bplt.jpg");
      plsdev("jpg");
      plscolbg(255,255,255);
      plsdidev(fmargin,aspect,yjust,xjust);
      plsfnam(pltfil);   
#ifdef PLPNEW
      plwidth(1.0);
#else
      plwid(1);
#endif
      break;}

       	      case 4:
      {
      plssub(1,1);
      plsdiori(iorient);
//      xaxis=-1.0;
      sprintf(pltfil,"bplt.pdf");
      plsdev("pdf");
      plscolbg(255,255,255);
      plsdidev(fmargin,aspect,yjust,xjust);
      plsfnam(pltfil);  
#ifdef PLPNEW
      plwidth(4.0);
#else
      plwid(4);
#endif
      break;}

      } //endswitch
//...initialize plot
      plinit();
      plssub(1,1);
      plfontld(0);
  
//set fill pattern
      plpsty(2);  //have to call this first, before plpat
      inc[0]=900;
      del[0]=1000;
      inc[1]=0;
      del[1]=1000;
      plpat(1,inc,del); 
 
//...set environment, window and viewport
//      plenv(xmin,xmax,ymin,ymax,0,0);
      pladv(0);
      plschr(0,.60);
      plprec(0,0);
      plssym(0,.60);
      
    //scale plot
      if (xaxis>0.0 && yaxis>0.0)
      {
   /*   plgspa(&gxmin,&gxmax,&gymin,&gymax);
      fprintf(stdout,"\n --gxmin=%f gxmax=%f gymin=%f gymax=%f",
		      gxmin,gxmax,gymin,gymax);  */
      xaxis=xaxis*25.4;
      yaxis=yaxis*25.4;
      plsvpa(25.4,25.4+(PLFLT)xaxis,25.4,25.4+(PLFLT)yaxis);
      }
      else
      {
      plvpor(0.1, .95, 0.10, .95);
      } //endif
      plwind((PLFLT)minx,(PLFLT)maxx,(PLFLT)smin,(PLFLT)smax);

	if (idev==2)    plcol0(9);  //use color 9
      plbox("cbgmst",0.,0.,"bcgnstv",0.,0.);

//set pen width
      //plwid(4);

//...write labels and title
      sprintf(titl," ");
      sprintf(titl2," [amp=%.2E percnt=%.0f %s]",amp,100*percnt,infile);
//...set label of non-time axis
      switch(itype)
      {
	      case 0:
	      {
      		sprintf(xlbl,"Trace Number  %s",titl2);
	       break;
	      } // 0 esac

	      case 1:
	      {
      		sprintf(xlbl,"Offset  %s",titl2);
	       break;
	      } // 1 esac

	      case 2:
	      {
      		sprintf(xlbl,"Geophone Elevation  %s",titl2);
	       break;
	      } // 2 esac

	      case 3:
	      {
      		sprintf(xlbl,"Geophone X-Coordinate  %s",titl2);
	       break;
	      } // 3 esac

	      case 4:
	      {
      		sprintf(xlbl,"Geophone Y-Coordinate  %s",titl2);
	       break;
	      } // 4 esac

	      case 5:
	      {
      		sprintf(xlbl,"Shot Elevation  %s",titl2);
	       break;
	      } // 5 esac

	      case 6:
	      {
      		sprintf(xlbl,"Shot X-Coordinate  %s",titl2);
	       break;
	      } // 6 esac

	      case 7:
	      {
      		sprintf(xlbl,"Shot Y-Coordinate  %s",titl2);
	       break;
	      } // 7 esac
      } //endswitch   

      sprintf(ylbl,"Time (ms)");
      pllab(xlbl,ylbl,titl);
      plcol0(15);
      if(idev==2 || idev==4)
      {
	      plscol0(9,0,0,0); //set color 9=black
	      plcol0(9);  //use color 9
      } //endif
      if(idev==3)
      {
	      plcol0(1);  //use color red
      } //endif


	} //endif 
//...plot the trace 
//WIGGLE TRACE
	if (istyl==0)	
	{
      		plline(npts,x,s);
	} // endif istyl==0 
	
//VARIABLE AREA (peaks black)	
	if (istyl==1)
	{
	j=1;  // polygon looks back to j-1, so start from j=1
	x[0]=(vzerod-x[1])+vzerod;  //hack to avoid infinite slope an Variable Area
	fsgn=1.0;
	if(iflip==1) fsgn=-1.0;
	while (j<npts-2)
	{ 
		if ( fsgn*(x[j]-vzerod)>=0.0 )
		{
			k=j;
			while (fsgn*(x[k]-vzerod)>=0.0 && k<(npts-2))
			{
				k=k+1;
			} //endwhile
			k=k-1;
			kpts=k-j+1;
	/*		 fprintf(stdout,"\n positive kpts=%d ",kpts);
			fflush(stdout); */
			if (kpts>1) 
			{

				//solve for polygon fill endpoints
				svx0=x[j-1];
				svy0=s[j-1];
				svx1=x[j+kpts];
				svy1=s[j+kpts];
				/* adjust to zero crossing if slope finite */
				  if (x[j]-x[j-1] != (PLFLT) 0.00) {
				  slope0=(s[j]-s[j-1])/(x[j]-x[j-1]);
				  s[j-1]=fabs(slope0*(x[j-1]-vzerod))+s[j-1];
				  } //endif

				  if (x[j+kpts]-x[j+kpts-1] != (PLFLT) 0.00) {
				  slope1=(s[j+kpts]-s[j+kpts-1])/(x[j+kpts]-x[j+kpts-1]);
				  s[j+kpts]=fabs(slope1*(x[j+kpts-1]-vzerod))+s[j+kpts-1];
				  } //endif

				/* avoid going out of bounds */
  			  	if (s[j+kpts]>smin )
					s[j+kpts]=smin;
				if (s[j-1]<smax )
					s[j-1]=smax;

			  	/* set plot to zero crossings */	  
				x[j-1]=vzerod;
				x[j+kpts]=vzerod;

				/* plot fill polygon in positive region */
				pm_plfill(&x[0],&s[0],kpts,j,vzerod,0);

				/* restore to original values */
				x[j-1]=svx0;
				s[j-1]=svy0;
				x[j+kpts]=svx1;
				s[j+kpts]=svy1;
			}
			else
				plline(kpts,&x[j],&s[j]);
			}
		else
		{
			k=j;
			while (fsgn*(x[k]-vzerod)<0.0 && k<npts-2)
			{
				k=k+1;
			} //endwhile
			k=k-1;
			kpts=k-j+1;
			/* fprintf(stdout,"\n negative kpts=%d ",kpts);
			fflush(stdout); */
			plline(kpts+2,&x[j-1],&s[j-1]);
		} //endif

		j=j+kpts;
	} //endwhile next j
      		plline(npts,x,s);  //draw wiggle trace on top so no blank areas
	} // endif istyl==1

//VARIABLE AREA Peaks Black, Trough Grey
	if (istyl==2)
	{
	j=1;  // polygon looks to j-1, so start from j=1
	x[0]=(vzerod-x[1])+vzerod;  //hack to avoid infinite slope an Variable Area
	fsgn=1.0;
	if(iflip==1) fsgn=-1.0;
	while (j<npts-2)
	{ 
		if ( fsgn*(x[j]-vzerod)>=0.0 )
		{
			k=j;
			while (fsgn*(x[k]-vzerod)>=0.0 && k<(npts-2))
			{
				k=k+1;
			} //endwhile
			k=k-1;
			kpts=k-j+1;
			/* fprintf(stdout,"\n positive kpts=%d ",kpts);
			fflush(stdout);*/
			if (kpts>1) 
			{
				//solve for polygon fill endpoints
				svx0=x[j-1];
				svy0=s[j-1];
				svx1=x[j+kpts];
				svy1=s[j+kpts];
				/* adjust to zero crossing if slope finite */
				  if (x[j]-x[j-1] != (PLFLT) 0.00) {
				  slope0=(s[j]-s[j-1])/(x[j]-x[j-1]);
				  s[j-1]=fabs(slope0*(x[j-1]-vzerod))+s[j-1];
				  } //endif

				  if (x[j+kpts]-x[j+kpts-1] != (PLFLT) 0.00) {
				  slope1=(s[j+kpts]-s[j+kpts-1])/(x[j+kpts]-x[j+kpts-1]);
				  s[j+kpts]=fabs(slope1*(x[j+kpts-1]-vzerod))+s[j+kpts-1];
				  } //endif

				/* avoid going out of bounds */
  			  	if (s[j+kpts]>smin )
					s[j+kpts]=smin;
				if (s[j-1]<smax )
					s[j-1]=smax;
			  		  
			  	/* set plot to zero crossings */	  
				x[j-1]=vzerod;
				x[j+kpts]=vzerod;

				/* plot fill polygon in positive region */
				  pm_plfill(&x[0],&s[0],kpts,j,vzerod,1);

				/* restore to original values */
				x[j-1]=svx0;
				s[j-1]=svy0;
				x[j+kpts]=svx1;
				s[j+kpts]=svy1;
			}
			else
				plline(kpts,&x[j],&s[j]);
			}
		else
		{
			k=j;
			while (fsgn*(x[k]-vzerod)<0.0 && k<npts-2)
			{
				k=k+1;
			} //endwhile
			k=k-1;
			kpts=k-j+1;
			/* fprintf(stdout,"\n negative kpts=%d ",kpts);
			fflush(stdout); */
			if (kpts>1) 
			{
			    /*    inc[0]=000;
      				del[0]=400;
      				inc[1]=0;
      				del[1]=400;
      				plpat(1,inc,del); 
      				plwid(1);
				plstyl(1,&mark,&space);*/

				//solve for polygon fill endpoints
				svx0=x[j-1];
				svy0=s[j-1];
				svx1=x[j+kpts];
				svy1=s[j+kpts];
				/* adjust to zero crossing if slope finite */
				  if (x[j]-x[j-1] != (PLFLT) 0.00) {
				  slope0=(s[j]-s[j-1])/(x[j]-x[j-1]);
				  s[j-1]=fabs(slope0*(x[j-1]-vzerod))+s[j-1];
				  } //endif

				  if (x[j+kpts]-x[j+kpts-1] != (PLFLT) 0.00) {
				  slope1=(s[j+kpts]-s[j+kpts-1])/(x[j+kpts]-x[j+kpts-1]);
				  s[j+kpts]=fabs(slope1*(x[j+kpts-1]-vzerod))+s[j+kpts-1];
				  } //endif

				/* avoid going out of bounds */
  			  	if (s[j+kpts]>smin )
					s[j+kpts]=smin;
				if (s[j-1]<smax )
					s[j-1]=smax;
			  		  
			  	/* set plot to zero crossings */	  
				x[j-1]=vzerod;
				x[j+kpts]=vzerod;

				/* plot fill polygon in positive region */
				  pm_plfill(&x[0],&s[0],kpts,j,vzerod,2);

				/* restore to original values */
				x[j-1]=svx0;
				s[j-1]=svy0;
				x[j+kpts]=svx1;
				s[j+kpts]=svy1;
			} 
			else
      		/*		plwid(4);
				plstyl(0,&mark,&space); */
				plline(kpts,&x[j],&s[j]);

			plline(kpts+2,&x[j-1],&s[j-1]);
		} //endif

		j=j+kpts;
	} //endwhile next j
	} // endif istyl==2

      
//      plend();
      return(0);
     }  //end tplot
	void pm_plfill(PLFLT x[], PLFLT s[], PLINT kpts, PLINT j,
	               PLFLT vzerod, PLINT fstyl)
	{
		PLINT	j2,jstep,nstep,jtmp,maxpoly;
		PLFLT	svx2,svy2,svx3;
		PLINT   nlin,inc[2],del[2];
		//maximum points in polygon
		maxpoly=200;  //library sets this to 256
		j2=j;
		nstep=floor(kpts/maxpoly);
		//set fill pattern
		nlin=1;
		inc[0]=0;
		del[0]=100;
		inc[1]=450;
		del[1]=700;
		if(fstyl==0)  //black peak if white trough
		{
#ifdef PLPNEW
		plwidth(1.0);
#else 
                plwid(1);
#endif 
      		plpsty(fstyl);
		}
		if(fstyl==1)  //black peak if grey trough
		{
#ifdef PLPNEW
		plwidth(1.0);
#else 
                plwid(1);
#endif 
		plpat(nlin,&inc[0],&del[0]);
		}
		if(fstyl==2)  //grey trough
		{
#ifdef PLPNEW
		plwidth(1.0);
#else 
                plwid(1);
#endif 
		plpat(nlin,&inc[1],&del[1]);
		} //endif 

		//avoid exceeding maxpoly
		if(nstep>0)
		{
		  svx3=x[j2-1];
		  for (jstep=0;jstep<nstep;jstep++)
	  	  {
		    jtmp=j2-1+maxpoly+1;
		    svx2=x[jtmp];
		    svy2=s[jtmp];
		    x[jtmp]=vzerod;   
		    s[jtmp]=s[jtmp-1];
		    plfill(maxpoly+2,&x[j2-1],&s[j2-1]);
		    x[j2-1]=svx3;
		    x[jtmp]=svx2;
		    s[jtmp]=svy2;
		    j2=j2-1+maxpoly;
		    svx3=x[j2-1];
		    x[j2-1]=vzerod;   
		  } //next jstep
		    plfill((j+kpts-j2+2),&x[j2-1],&s[j2-1]);
		    x[j2-1]=svx3;
		 }  
		  else
		 {
		    plfill(kpts+2,&x[j-1],&s[j-1]);
		 }
	} //end pm_plfill
#else
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: genplotA                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: plots signal using pipe to gnuplot                |
   |                                                            |
   |------------------------------------------------------------|
*/
	void genplotA (float tmin, float tmax, char infile[], FILE *pipe,
	 int itype, float minx, float maxx, float dx, int idev, char pltfil[],
	 float amp, float xaxis, float yaxis)
	{
	char	YL[12]="Time (ms)";
	char	XL[30];
	char	XR[55];
	int	imin,imax;
	float	fmin,fmax;

	if (xaxis < 0.0 ) xaxis=6.0;
	if (yaxis < 0.0 ) yaxis=4.0;

	switch (itype) {
		case 0: {
		sprintf(XL,"Trace Number");
		imin= floor(minx-.5);
		imax=ceil(maxx+.5);
		sprintf(XR,"set xrange [%d:%d]",imin,imax); 
		break;}
		case 1:  {
		imin= floor(minx-1.0);
		imax=ceil(maxx+1.0);
		sprintf(XL,"Offset ");
		fmin = (minx < maxx) ? minx : maxx;
		fmax = (maxx > minx) ? maxx : minx;
		sprintf(XR,"set xrange [%.1f:%.1f]",fmin-dx,fmax+dx); 
		break;}
		case 2:  {
		sprintf(XL,"Geophone Z-Coordinate");
		fmin = (minx < maxx) ? minx : maxx;
		fmax = (maxx > minx) ? maxx : minx;
		sprintf(XR,"set xrange [%.1f:%.1f]",fmin-dx,fmax+dx); 
		break;}
		case 3:  {
		sprintf(XL,"Geophone X-Coordinate");
		fmin = (minx < maxx) ? minx : maxx;
		fmax = (maxx > minx) ? maxx : minx;
		sprintf(XR,"set xrange [%.1f:%.1f]",fmin-dx,fmax+dx); 
		break;}
		case 4:  {
		sprintf(XL,"Geophone Y-Coordinate");
		fmin = (minx < maxx) ? minx : maxx;
		fmax = (maxx > minx) ? maxx : minx;
		sprintf(XR,"set xrange [%.1f:%.1f]",fmin-dx,fmax+dx); 
		break;}
		case 5:  {
		sprintf(XL,"Shot Z-Coordinate");
		fmin = (minx < maxx) ? minx : maxx;
		fmax = (maxx > minx) ? maxx : minx;
		sprintf(XR,"set xrange [%.1f:%.1f]",fmin-dx,fmax+dx); 
		break;}
		case 6:  {
		sprintf(XL,"Shot X-Coordinate");
		fmin = (minx < maxx) ? minx : maxx;
		fmax = (maxx > minx) ? maxx : minx;
		sprintf(XR,"set xrange [%.1f:%.1f]",fmin-dx,fmax+dx); 
		break;}
		case 7:  {
		sprintf(XL,"Shot Y-Coordinate");
		fmin = (minx < maxx) ? minx : maxx;
		fmax = (maxx > minx) ? maxx : minx;
		sprintf(XR,"set xrange [%.1f:%.1f]",fmin-dx,fmax+dx); 
		break;}

	} //end switch

	#ifdef WIN
	char Xwin[4]="wxt";
	#else
	char Xwin[4]="x11";
	#endif
//	FILE *pipe = popen("gnuplot -p","w");
  	fprintf(pipe,"set grid \n");
  //	fprintf(pipe,"set key left \n");
  	fprintf(pipe,"set nokey \n");
  	fprintf(pipe,"set xlabel \" %s \" \n",XL);
  	fprintf(pipe,"set ylabel \" %s \" \n",YL);
  	fprintf(pipe,"set title \"File: %s amp=%10.3E  \"  \n",infile,amp);
        fprintf(pipe,"set yrange [%f:%f] reverse \n",tmin,tmax);
	fprintf(pipe,"%s \n",XR);
	fprintf(pipe,"set style line 1 lc rgb 'black' lt 1 lw 1 \n");
	switch (idev) {
		case 0: {
		fprintf(pipe,"set terminal %s persist \n",Xwin);
		break;}

		case 1: {
		fprintf(pipe,"set terminal postscript enhanced color size %f,%f in\n",
		xaxis,yaxis);
		fprintf(pipe,"set output '%s' \n",pltfil);
		break;}

		case 2: {
		fprintf(pipe,"set terminal fig monochrome size %f,%f inches \n",
		xaxis,yaxis);
		fprintf(pipe,"set output '%s' \n",pltfil);
		break;}

		case 3: {
		fprintf(pipe,"set terminal jpeg enhanced nocrop size %f,%f \n",
		xaxis*100,yaxis*100);  //units pixels
		fprintf(pipe,"set output '%s' \n",pltfil);
		break;}

		case 4: {
		fprintf(pipe,"set terminal pdf color enhanced size %f,%f in \n",
		xaxis,yaxis);  
		fprintf(pipe,"set output '%s' \n",pltfil);
		break;}
	} //endswitch
	} //end genplotA
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: genplotA                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: plots signal using pipe to gnuplot                |
   |                 Landscape version                          |
   |------------------------------------------------------------|
*/
	void genplotAL (float tmin, float tmax, char infile[], FILE *pipe,
	 int itype, float minx, float maxx, float dx, int idev, char pltfil[],
	 float amp, float xaxis, float yaxis)
	{
	char	XL[12]="Time (ms)";
	char	YL[30];
	char	YR[55];
	int	imin,imax;
	float	fmin,fmax;

	if (xaxis < 0.0 ) xaxis=6.0;
	if (yaxis < 0.0 ) yaxis=4.0;

	switch (itype) {
		case 0: {
		sprintf(YL,"Trace Number");
		imin= floor(minx-.5);
		imax=ceil(maxx+.5);
		sprintf(YR,"set yrange [%d:%d]",imin,imax); 
		break;}
		case 1:  {
		imin= floor(minx-1.0);
		imax=ceil(maxx+1.0);
		sprintf(YL,"Offset ");
		fmin = (minx < maxx) ? minx : maxx;
		fmax = (maxx > minx) ? maxx : minx;
		sprintf(YR,"set yrange [%.1f:%.1f]",fmin-dx,fmax+dx); 
		break;}
		case 2:  {
		sprintf(YL,"Geophone Z-Coordinate");
		fmin = (minx < maxx) ? minx : maxx;
		fmax = (maxx > minx) ? maxx : minx;
		sprintf(YR,"set yrange [%.1f:%.1f]",fmin-dx,fmax+dx); 
		break;}
		case 3:  {
		sprintf(YL,"Geophone X-Coordinate");
		fmin = (minx < maxx) ? minx : maxx;
		fmax = (maxx > minx) ? maxx : minx;
		sprintf(YR,"set yrange [%.1f:%.1f]",fmin-dx,fmax+dx); 
		break;}
		case 4:  {
		sprintf(YL,"Geophone Y-Coordinate");
		fmin = (minx < maxx) ? minx : maxx;
		fmax = (maxx > minx) ? maxx : minx;
		sprintf(YR,"set yrange [%.1f:%.1f]",fmin-dx,fmax+dx); 
		break;}
		case 5:  {
		sprintf(YL,"Shot Z-Coordinate");
		fmin = (minx < maxx) ? minx : maxx;
		fmax = (maxx > minx) ? maxx : minx;
		sprintf(YR,"set yrange [%.1f:%.1f]",fmin-dx,fmax+dx); 
		break;}
		case 6:  {
		sprintf(YL,"Shot X-Coordinate");
		fmin = (minx < maxx) ? minx : maxx;
		fmax = (maxx > minx) ? maxx : minx;
		sprintf(YR,"set yrange [%.1f:%.1f]",fmin-dx,fmax+dx); 
		break;}
		case 7:  {
		sprintf(YL,"Shot Y-Coordinate");
		fmin = (minx < maxx) ? minx : maxx;
		fmax = (maxx > minx) ? maxx : minx;
		sprintf(YR,"set yrange [%.1f:%.1f]",fmin-dx,fmax+dx); 
		break;}

	} //end switch

	#ifdef WIN
	char Xwin[4]="wxt";
	#else
	char Xwin[4]="x11";
	#endif
//	FILE *pipe = popen("gnuplot -p","w");
  	fprintf(pipe,"set grid \n");
  //	fprintf(pipe,"set key left \n");
  	fprintf(pipe,"set nokey \n");
  	fprintf(pipe,"set xlabel \" %s \" \n",XL);
  	fprintf(pipe,"set ylabel \" %s \" \n",YL);
  	fprintf(pipe,"set title \"File: %s amp=%10.3E  \"  \n",infile,amp);
        fprintf(pipe,"set xrange [%f:%f] \n",tmax,tmin);
	fprintf(pipe,"%s \n",YR);
	fprintf(pipe,"set style line 1 lc rgb 'black' lt 1 lw 1 \n");
	switch (idev) {
		case 0: {
		fprintf(pipe,"set terminal %s persist \n",Xwin);
		break;}

		case 1: {
		fprintf(pipe,"set terminal postscript enhanced color size %f,%f in\n",
		xaxis,yaxis);
		fprintf(pipe,"set output '%s' \n",pltfil);
		break;}

		case 2: {
		fprintf(pipe,"set terminal fig monochrome size %f,%f inches \n",
		xaxis,yaxis);
		fprintf(pipe,"set output '%s' \n",pltfil);
		break;}

		case 3: {
		fprintf(pipe,"set terminal jpeg enhanced nocrop size %f,%f \n",
		xaxis*100,yaxis*100);  //units pixels
		fprintf(pipe,"set output '%s' \n",pltfil);
		break;}

		case 4: {
		fprintf(pipe,"set terminal pdf color enhanced size %f,%f in \n",
		xaxis,yaxis);  
		fprintf(pipe,"set output '%s' \n",pltfil);
		break;}
	} //endswitch

	} //end genplotAL
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: genplotB                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: plots signal using pipe to gnuplot                |
   |                                                            |
   |------------------------------------------------------------|
*/
	void genplotB (int ntraces, FILE *pipe, int istyl, float x[] )
	{
	int itrace;

	switch (istyl)
	{
		case 0: {
	if (ntraces > 1 ) {
  	fprintf(pipe," plot '-' u ($2):($1) ls 1 w l ,");
	fprintf(pipe," '' u ($2):($1) ls 1 w l , \\\n");}
	else {
  	fprintf(pipe," plot '-' u ($2):($1) ls 1 w l , ");
	fprintf(pipe," '' u ($2):($1) ls 1 w l  \n"); }

	for (itrace=1; itrace<ntraces-1; itrace++)
	{
  	fprintf(pipe,"  '' u ($2):($1) ls 1 w l ," );
	fprintf(pipe," '' u ($2):($1) ls 1 w l , \\\n");
	}  //next itrace
  	fprintf(pipe,"  '' u ($2):($1) ls 1 w l , ");
	fprintf(pipe," '' u ($2):($1) ls 1 w l  \n");
	break; }

		case 1: {
	if (ntraces > 1 ) {
  	fprintf(pipe," plot '-' u ($2):($1) ls 1 w filledc above x1=%f ,",x[0]);
	fprintf(pipe," '' u ($2):($1) ls 1 w l , \\\n");}
	else {
  	fprintf(pipe," plot '-' u ($2):($1) ls 1 w filledc above x1=%f , ",x[0]);
	fprintf(pipe," '' u ($2):($1) ls 1 w l  \n"); }

	for (itrace=1; itrace<ntraces-1; itrace++)
	{
  	fprintf(pipe,"  '' u ($2):($1) ls 1 w filledc above x1=%f ,",x[itrace]);

	fprintf(pipe," '' u ($2):($1) ls 1 w l , \\\n");
	}  //next itrace
  	fprintf(pipe,"  '' u ($2):($1) ls 1 w filledc above x1=%f, ",x[itrace]);
	fprintf(pipe," '' u ($2):($1) ls 1 w l  \n");
	break; }
		case 2: {
	if (ntraces > 1 ) {
  	fprintf(pipe," plot '-' u ($2):($1) ls 1 w filledc above x1=%f ,",x[0]);
	fprintf(pipe," '' u ($2):($1) ls 1 w filledc below x1=%f fills s .3 , \\\n",x[0]);}
	else {
  	fprintf(pipe," plot '-' u ($2):($1) ls 1 w filledc above x1=%f , ",x[0]);
	fprintf(pipe," '' u ($2):($1) ls 1 w filledc below x1=%f fills s .3  \n",x[0]); }

	for (itrace=1; itrace<ntraces-1; itrace++)
	{
  	fprintf(pipe,"  '' u ($2):($1) ls 1 w filledc above x1=%f ,",x[itrace]);
	fprintf(pipe," '' u ($2):($1) ls 1 w filledc below x1=%f fills s .3 , \\\n",x[itrace]);
	}  //next itrace
  	fprintf(pipe,"  '' u ($2):($1) ls 1 w filledc above x1=%f, ",x[itrace]);
	fprintf(pipe," '' u ($2):($1) ls 1 w filledc below x1=%f fills s .3 \n",x[itrace]);
	break; }

	} // endcase style

	} //end genplotB
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: genplotB                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: plots signal using pipe to gnuplot                |
   |                 Landscape version                          |
   |------------------------------------------------------------|
*/
	void genplotBL (int ntraces, FILE *pipe, int istyl, float x[] )
	{
	int itrace;

	switch (istyl)
	{
		case 0: {
	if (ntraces > 1 ) {
  	fprintf(pipe," plot '-' u ($1):($2) ls 1 w l ,");
	fprintf(pipe," '' u ($1):($2) ls 1 w l , \\\n");}
	else {
  	fprintf(pipe," plot '-' u ($1):($2) ls 1 w l , ");
	fprintf(pipe," '' u ($1):($2) ls 1 w l  \n"); }

	for (itrace=1; itrace<ntraces-1; itrace++)
	{
  	fprintf(pipe,"  '' u ($1):($2) ls 1 w l ," );
	fprintf(pipe," '' u ($1):($2) ls 1 w l , \\\n");
	}  //next itrace
  	fprintf(pipe,"  '' u ($1):($2) ls 1 w l , ");
	fprintf(pipe," '' u ($1):($2) ls 1 w l  \n");
	break; }

		case 1: {
	if (ntraces > 1 ) {
  	fprintf(pipe," plot '-' u ($1):($2) ls 1 w filledc above y1=%f ,",x[0]);
	fprintf(pipe," '' u ($1):($2) ls 1 w l , \\\n");}
	else {
  	fprintf(pipe," plot '-' u ($1):($2) ls 1 w filledc above y1=%f , ",x[0]);
	fprintf(pipe," '' u ($1):($2) ls 1 w l  \n"); }

	for (itrace=1; itrace<ntraces-1; itrace++)
	{
  	fprintf(pipe,"  '' u ($1):($2) ls 1 w filledc above y1=%f ,",x[itrace]);

	fprintf(pipe," '' u ($1):($2) ls 1 w l , \\\n");
	}  //next itrace
  	fprintf(pipe,"  '' u ($1):($2) ls 1 w filledc above y1=%f, ",x[itrace]);
	fprintf(pipe," '' u ($1):($2) ls 1 w l  \n");
	break; }
		case 2: {
	if (ntraces > 1 ) {
  	fprintf(pipe," plot '-' u ($1):($2) ls 1 w filledc above y1=%f ,",x[0]);
	fprintf(pipe," '' u ($1):($2) ls 1 w filledc below y1=%f fills s .3 , \\\n",x[0]);}
	else {
  	fprintf(pipe," plot '-' u ($1):($2) ls 1 w filledc above y1=%f , ",x[0]);
	fprintf(pipe," '' u ($1):($2) ls 1 w filledc below y1=%f fills s .3  \n",x[0]); }

	for (itrace=1; itrace<ntraces-1; itrace++)
	{
  	fprintf(pipe,"  '' u ($1):($2) ls 1 w filledc above y1=%f ,",x[itrace]);
	fprintf(pipe," '' u ($1):($2) ls 1 w filledc below y1=%f fills s .3 , \\\n",x[itrace]);
	}  //next itrace
  	fprintf(pipe,"  '' u ($1):($2) ls 1 w filledc above y1=%f, ",x[itrace]);
	fprintf(pipe," '' u ($1):($2) ls 1 w filledc below y1=%f fills s .3 \n",x[itrace]);
	break; }

	} // endcase style

	} //end genplotBL

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: genplot                                     |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: plots signal using pipe to gnuplot                |
   |                                                            |
   |------------------------------------------------------------|
*/
	void genplot (float tm[], float y[], int itrace, int npts, FILE *pipe )
	{
  	int j;
//	float slevel;
//	slevel=(float) (itrace + 1);
  		for(j=0;j<npts;j++) {
    		fprintf(pipe,"%14.7E %14.7E \n",tm[j],y[j]);
  		} //next j 
  	fprintf(pipe,"e\n");  //this is required when std in - plot
 		for(j=0;j<npts;j++) {
    		fprintf(pipe,"%14.7E %14.7E \n",tm[j],y[j]);
  		} //next j 
  	fprintf(pipe,"e\n");  //this is required when std in - plot
	} //end genplot
#endif
