// |$Id: tplt.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $              |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: tplt.c                                      |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: plot a trace from a BSEGY formated file           |
   |          Requires gnuplot                                  |
   |          Uses a pipe to plot selected trace                |
   |------------------------------------------------------------|
   |This version solves problem if compiled in a Gnuplot5 env.  |
   |                                                            |
   | NOTE: Revised for both Gnuplot 4 and 5                     |
   | Gnuplot 5 is buggy! While Gnuplot 4 will simply plot filled|
   | peaks with a simple plot '-' u $1:$2 w filledc above y1=0  |
   | Gnuplot 5 requires a zero data column to replace y1=0      |
   | and then a second repeat of first two columns for a replot |
   | of the curve itself for negative values.                   |
   |EXAMPLE OF PROBLEM (consider alternatives)                  |
   |plot "+" u 1:(sin($1)) w filledc above y1=0                 |
   |plot "+" u 1:(sin($1)):(0) w filledc above                  |
   |plot "+" u 1:(sin($1)):(0) w filledc above, '' u 1:(sin($1)) w l
   |                                                            |
   |The first command works in gnuplot 4                        |
   |The third command works in gnuplot 4 or 5                   |
   |The second command does not plot the curve below y1=0       |
   |                                                            |
   |SOLUTION: Conditional compile for gnuplot5 environment      |
   | If GNUPLOT5 is defined, generates extra code for pipe      |
   | and output file, ELSE will compile for gnuplot4            |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/c_bsegy.h"

//...parameter definitions

//...function prototypes
      #include "../include/sub4.h"

      int  getparm(int argc,char *argv[],
        int *itrace, float *tmin, float *tmax, char infile[],
        char ofile[], char lstfile[], char pid[]);

      void outlst(int itrace, char infile[], float tmin, float tmax,
        char pid[], char ofile[], char lstfile[], FILE *h3);

      void genplot (float tm[], float y[], int npts,
	float tmin, float tmax, char infile[], int itrace );

      void saveplot (FILE * h2, float tm[], float y[], int npts,
	float tmin, float tmax, char infile[], int itrace );

      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h2, *h3;
      int               jt,
                        jrec,
                        npts,
                        ntraces,
                        ibar,
                        invbar;
      float             fsamin,
                        bar,
			tmin,
			tmax,
			*tm,
                        *s1;
      char              pid[5]="tplt",   //process id name (4char)
                        ofile[13],
                        lstfile[13],
                        infile[14];
      struct header     hd;

//...specific to this application
        int             itrace;
	float 		fmin, fmax;
	int   		kmx, kmn, isw=0;
	float		tmaxdata;

//...get parameters
      if(getparm(argc,argv,&itrace,&tmin,&tmax,
         infile,ofile,lstfile,pid)!=0)
      {
       goto quick_exit;
      }


//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",infile);
          goto exit_3;
        }

//...open output file
        if( (h2=fopen(ofile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",ofile);
          goto exit_1;
        }

//...check input file size
          if(in_chk(&ntraces,&npts,&fsamin,hd,h1)!=0)
         {
          goto exit_1;
         }

//...check range of tmin and tmax
	tmaxdata = fsamin*npts;
	if (tmax > tmaxdata) tmax = tmaxdata;
	if (tmin < 0.0 ) tmin = 0.0;
	int  nptsplt= (int) (tmax/fsamin);
//...set trace number as if 1 is first trace
	itrace = itrace -1;
	if (itrace < 0 ) itrace=0;
	if (itrace > ntraces-1) itrace = ntraces-1;
//...echo check to listing file
      outlst(itrace+1,infile,tmin,tmax,pid,ofile,lstfile,h3);

//...dynamic allocation of seimic trace memory
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          fprintf(stderr,"ABORT:s1 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_1;
         }
//...dynamic allocation time vector memory
        tm=calloc(npts,sizeof(float));
          if(tm==NULL)
         {
          fprintf(stderr,"ABORT:tm can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_1;
         }

//-----------------main loop---------------------------
//...display progress bar grid

      bargrid(ntraces,&bar,&ibar,&invbar);

//...trace loop


//...read trace
        if(c_bsegin(itrace,s1,npts,&hd,h1)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegin");
        goto quit_it;
       }

//...compute time vector
	for (jt=0; jt<npts; jt++) tm[jt]=( (float) (jt) )*fsamin;

//...find max and min values
	fmax_min(nptsplt,s1,&fmax,&fmin,isw,&kmx,&kmn);

//...print ranges of data
	fprintf(h3,"\n     TRACE STATISTICS");
	fprintf(h3,"\n     Data tmax=%.5f (s)",tmaxdata);
	fprintf(h3,"\n     Data min value =%+.7E at %.5f (s)",
				fmin,(kmn-1)*fsamin);
	fprintf(h3,"\n     Data max value =%+.7E at %.5f (s)",
				fmax,(kmx-1)*fsamin);

//...print input file data to lstfile
        fprintf(h3,"\n     Number of samples= %d",npts);
        fprintf(h3,"\n     Sample Interval = %5.4f sec.",fsamin);
        fprintf(h3,"\n     Number of traces = %d \n",ntraces);
	fprintf(h3,"\n     For interactive plot: gnuplot; load \"graph.gp\" \n");
fflush(h3);


//...plot trace
	genplot (tm,s1,nptsplt,tmin,tmax,infile,itrace+1 );

	saveplot (h2,tm,s1,nptsplt,tmin,tmax,infile,itrace+1 );

//...display progress bar
      for (jrec=0; jrec<ntraces; jrec++) {
      exbar(ntraces,bar,ibar,invbar,jrec+1);}

        free(s1);
        fclose(h1);
        fclose(h3);

      printf("\n   Listing ====>%s",lstfile);
      printf("\n   Output  ====>%s",ofile);
      printf("\n   Processed %d traces",ntraces);
      printf("\n For interactive plot, start gnuplot");
      printf("\n In gnuplot shell:  load \"graph.gp\" \n");
        return(0);

quit_it:
        puts(" ");
//exit_s1:
        free(s1);
exit_1:
        fclose(h1);
        fclose(h2);
exit_3:
        fclose(h3);
quick_exit:
        return(1);
     } //  end of main
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
         int *itrace, float *tmin, float *tmax, char infile[],
         char ofile[], char lstfile[],char pid[])
     {
      //char              sfx1[4]="seg";
      char                sfx2[4]="lst";
      char		help[3]="-h";
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"Plot a trace                                 ");
      sprintf(msg2,"Selected trace plotted with gnuplot          ");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  %s  infile  trace_number tmin tmax \n",pid);
      puts(" ");
      puts("     infile  = input file name (4char minimum)");
      puts("     trace_number = trace number to plot      ");
      puts("     tmin = minimum time in seconds ");
      puts("     tmax = maximum time in seconds ");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strncpy(infile,argv[1],13);
       else
      {
       puts(" Enter INPUT file name ");
       scanf(" %13s",infile);
      }
	if (namecheck(12,infile) != 0 ) return (1);

//...convert string to integer
      if(argc>2)
       *itrace=atoi(argv[2]);
       else
      {
       puts(" Enter trace number (int)");
       scanf(" %d",itrace);
      }
//...convert string to float  
      if(argc>3)
       *tmin=atof(argv[3]);
       else
      {
       puts(" Enter tmin ");
       scanf(" %f",tmin);
      }
//...convert string to float  
      if(argc>4)
       *tmax=atof(argv[4]);
       else
      {
       puts(" Enter tmax ");
       scanf(" %f",tmax);
      }
//...name output files
//      names(infile,pid,ofile,sfx1);
	sprintf(ofile,"graph.gp");
      names(infile,pid,lstfile,sfx2);
       return(0);
     }  //  endif bhelp test
      } // end getparm
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(int itrace,char infile[], float tmin, float tmax,
        char pid[], char ofile[], char lstfile[], FILE *h3)
     {

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %s.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Input   File= %s",infile);
//      fprintf(h3,"\n     Output  File= %s",ofile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fprintf(h3,"\n     trace number= %5.5d ",itrace);
      fprintf(h3,"\n     tmin = %.5f (s)     ",tmin);
      fprintf(h3,"\n     tmax = %.5f (s)     ",tmax);
      fputs("\n",h3);                      //blank line
     }  //  end outlst

//CONDITIONAL COMPILE for gnuplot 5 or ELSE gnuplot 4 further below
	#ifdef GNUPLOT5
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: genplot                                     |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: plots signal using pipe to gnuplot                |
   | NOTE: Revised for both Gnuplot 4 and 5                     |
   | Gnuplot 5 is buggy! While Gnuplot 4 will simply plot filled|
   | peaks with a simple plot '-' u $1:$2 w filledc above y1=0  |
   | Gnuplot 5 requires a zero data column to replace y1=0      |
   | and then a second repeat of first two columns for a replot |
   | of the curve itself for negative values.                   |
   |------------------------------------------------------------|
*/
	void genplot (float tm[], float y[], int npts,
	float tmin, float tmax, char infile[], int itrace )
	{
	char	XL[9]="Time (s)";
	#ifdef WIN
	char Xwin[4]="wxt";
	#else
	char Xwin[4]="x11";
	#endif
	float ybase=0.0;
	FILE *pipe = popen("gnuplot ","w");
  	fprintf(pipe,"set grid \n");
  	fprintf(pipe,"set key left \n");
  	fprintf(pipe,"set xlabel \" %s \" \n",XL);
  	fprintf(pipe,"set title \" %s \"  \n",infile);
        fprintf(pipe,"set xrange [%f:%f] \n",tmin,tmax);
	fprintf(pipe,"set terminal %s persist \n",Xwin);
  	fprintf(pipe," plot '-' u ($1):($2):($3) lc 'black' w filledc above t '' ,");
	fprintf(pipe," '' u ($1):($2) lc 'black' w l t \"  Trace %d \" \n",itrace);
  	int j;
  		for(j=0;j<npts;j++) {
    		fprintf(pipe,"%f %e %e \n",tm[j],y[j],ybase);
  		} //next j 
  	fprintf(pipe,"e\n");  //this is required when std in - plot
 		for(j=0;j<npts;j++) {
    		fprintf(pipe,"%f %e %e \n",tm[j],y[j],ybase);
  		} //next j 
  	fprintf(pipe,"e\n");  //this is required when std in - plot

//...close pipe
	fflush(pipe);
	getchar(); //wait for user input
	pclose(pipe);
	} //end genplot
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: genplot                                     |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: saves plot to a file                              |
   |                                                            |
   | NOTE: Revised for both Gnuplot 4 and 5                     |
   | Gnuplot 5 is buggy! While Gnuplot 4 will simply plot filled|
   | peaks with a simple plot '-' u $1:$2 w filledc above y1=0  |
   | Gnuplot 5 requires a zero data column to replace y1=0      |
   | and then a second repeat of first two columns for a replot |
   | of the curve itself for negative values.                   |
   |------------------------------------------------------------|
*/
	void saveplot (FILE * h2 , float tm[], float y[], int npts,
	float tmin, float tmax, char infile[], int itrace )
	{
	char	XL[9]="Time (s)";
	#ifdef WIN
	char Xwin[4]="wxt";
	#else
	char Xwin[4]="x11";
	#endif
	float ybase=0.0;
  	fprintf(h2,"set grid \n");
  	fprintf(h2,"set terminal %s persist \n",Xwin);
  	fprintf(h2,"set key left \n");
  	fprintf(h2,"set xlabel \" %s \" \n",XL);
  	fprintf(h2,"set title \" %s \"  \n",infile);
        fprintf(h2,"set xrange [%f:%f] \n",tmin,tmax);
  	fprintf(h2," plot '-' u ($1):($2):($3) lc 'black' w filledc above t '' ,");
	fprintf(h2," '' u ($1):($2) lc 'black' w l t \"  Trace %d \" \n",itrace);
  	int j;
  		for(j=0;j<npts;j++) {
    		fprintf(h2,"%f %e %e \n",tm[j],y[j],ybase);
  		} //next j 
  	fprintf(h2,"e\n");  //this is required when std in - plot
 		for(j=0;j<npts;j++) {
    		fprintf(h2,"%f %e %e \n",tm[j],y[j],ybase);
  		} //next j 
  	fprintf(h2,"e\n");  //this is required when std in - plot
	} //end saveplot
	#else
//  COMPILE FOR GNUPLOT4 follows
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: genplot                                     |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: plots signal using pipe to gnuplot                |
   |                                                            |
   |------------------------------------------------------------|
*/
	void genplot (float tm[], float y[], int npts,
	float tmin, float tmax, char infile[], int itrace )
	{
	char	XL[9]="Time (s)";
	#ifdef WIN
	char Xwin[4]="wxt";
	#else
	char Xwin[4]="x11";
	#endif
	FILE *pipe = popen("gnuplot ","w");
  	fprintf(pipe,"set grid \n");
  	fprintf(pipe,"set key left \n");
  	fprintf(pipe,"set xlabel \" %s \" \n",XL);
  	fprintf(pipe,"set title \" %s \"  \n",infile);
        fprintf(pipe,"set xrange [%f:%f] \n",tmin,tmax);
	fprintf(pipe,"set terminal %s persist \n",Xwin);
  	fprintf(pipe," plot '-' u ($1):($2) lc 'black' w filledc above y1=0 t \"Trace: %d \" \n",itrace);
  	int j;
  		for(j=0;j<npts;j++) {
    		fprintf(pipe,"%f %e \n",tm[j],y[j]);
  		} //next j 
  	fprintf(pipe,"e\n");  //this is required when std in - plot
//...close pipe
	fflush(pipe);
	getchar(); //wait for user input
	pclose(pipe);
	} //end genplot

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: genplot                                     |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: saves plot to a file                              |
   |                                                            |
   |------------------------------------------------------------|
*/
	void saveplot (FILE * h2 , float tm[], float y[], int npts,
	float tmin, float tmax, char infile[], int itrace )
	{
	char	XL[9]="Time (s)";
	#ifdef WIN
	char Xwin[4]="wxt";
	#else
	char Xwin[4]="x11";
	#endif
  	fprintf(h2,"set grid \n");
  	fprintf(h2,"set terminal %s persist \n",Xwin);
  	fprintf(h2,"set key left \n");
  	fprintf(h2,"set xlabel \" %s \" \n",XL);
  	fprintf(h2,"set title \" %s \"  \n",infile);
        fprintf(h2,"set xrange [%f:%f] \n",tmin,tmax);
  	fprintf(h2," plot '-' u ($1):($2) lc 'black' w filledc above y1=0  t \"Trace: %d \" \n",itrace);
  	int j;
  		for(j=0;j<npts;j++) {
    		fprintf(h2,"%f %e \n",tm[j],y[j]);
  		} //next j 
  	fprintf(h2,"e\n");  //this is required when std in - plot
	} //end saveplot
	#endif
