c |$Id: bfxt.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bfxt.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | Takes FX transform of seismic traces (time-->frequency),   |
c | The spatial variable remains untransformed.                |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |     DRUM    Modified E.A. Robinson's phase unwrapping sub  |
c |     RECT    Converts from Polar to Rectangular form        |
c |     NRAD2   Determines power of two number of samples      |
c |     CZERO   Zeros a complex signal                         |
c |     NLOGN   E.A. Robinson's radix 2 FFT                    |
c |     POLAR   E.A. Robinson's conversion;rectangular to polar|
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
      real*4            s1(ndim),s2(ndim),s3(ndim),xi(ndim),xr(ndim)
      character*40      infil , outfl2, outfl3, aplc, outlst
      character*40      infil2
      complex           x(ndim)
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      iunit3=3
      icall1=0
      icall2=0
      icall3=0
c
c...define application prefix
      write(aplc,'(''bfxt'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +iopt,infil2)                  
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...name outputfile
      if(iopt.ge.1) then
      write(outfl2,'(a,''ampl.seg'')') aplc(1:4)
      write(outfl3,'(a,''phaz.seg'')') aplc(1:4)
      else
      write(outfl2,'(a,''.seg'')') aplc(1:4)
      write(outfl3,'('' none   '')')            
      endif
c
c...compute radix 2
      call nrad2(npts,npr2)
      npr2h=npr2/2
      fsamin2=1.0/(npr2*fsamin)
c...decimate by iskip, scale to hz
      iskip=1
      fsamin2=fsamin2*(float(iskip)/1000.)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,outlst,outfl2,outfl3,
     +infil,iopt,infil2,fsamin2,npr2)
c
c...clear output file...
      open(10,file=outfl2,access='direct',recl=240)
      close(10,status='delete')
      open(10,file=outfl3,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
      ntr=0
c
c...data loop=======================================....
c
c------------IOPT=1----------------------
      if(iopt.ge.1) then
c
      do 100 jrec=1,ntrace
c
c...input a trace
      call BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
      call czero(npr2,x)
c
c...compute DFT
      sign=-1.0
      do 109 jt=1,npts
      x(jt)=cmplx(s1(jt),0.0)
 109  continue
      call nlogn(npr2,x,sign)
c
      npts2=0
      do 110 jt=1,npr2h,iskip
      npts2=npts2+1
      xi(npts2)=aimag(x(jt))
      xr(npts2)=real(x(jt))
 110  continue
      call polar(npts2,xr,xi,s2,s3)
c
c...unwrap phase function
      if(iopt.eq.1) then
c   (pivot from max amplitude point)
      fmax=-1.e+20
      do 112 jt=1,npts2
      if(s2(jt).ge.fmax) then
      fmax=s2(jt)
      jfmax=jt
      endif
 112  continue
      call drum(npts2,s3,jfmax)
      endif
c
c...output trace
      CALL BSEGOUT(iunit2,
     +icall2,npts2,s2,fsamin2,outfl2,jrec)
c
      CALL BSEGOUT(iunit3,
     +icall3,npts2,s3,fsamin2,outfl3,jrec)
c
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c
c...END LOOP========================================....
  100 continue
c---------------IOPT=-1
      else
c
      fsamin2=fsamin*1000.
      fsamin=1.0/(npts*fsamin2*2.0)
      write(io11,'('' Output sample interval='',
     1f10.5,'' sec'')') fsamin
      npts2=npts*2
      write(io11,'('' Number of samples out='',i5)') npts2
c===start loop
      do 200 jrec=1,ntrace
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s2,fsamin2,infil,jrec,iexit)
c
      ntr = ntr + 1
c
      CALL BSEGIN(iunit2,ndim,
     +icall2,npts,s3,fsamin2,infil2,jrec,iexit)
c
      call rect(npts,xr,xi,s2,s3)
      npts2=npts*2
      do 203 jt=1,npts
      x(jt)=cmplx(xr(jt),xi(jt))
      x(npts2-jt+2)=cmplx(xr(jt),-xi(jt))
 203  continue
      sign=1.0
      call nlogn(npts2,x,sign)
      do 215 jt=1,npts2
      xi(jt)=aimag(x(jt))
      xr(jt)=real(x(jt))
 215  continue

c...output trace
      CALL BSEGOUT(iunit3,
     +icall3,npts2,xr,fsamin,outfl2,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c...END LOOP========================================....
  200 continue
c
      endif
c----------------endif of IOPT 
c
      write(*,*) '  '
      write(6,240) ntr,outfl2,outfl3,outlst
240   format(/' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> amplitudes in ',a,
     +/'====> listing in ',a)
c
c
      close(io11)
      close(iunit1)
      close(iunit2)
      close(iunit3)
      stop
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +iopt,infil2)                  
c
      character*40      infil,infil2
      character*40      arg1
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bfxt: FX Transform of seismic data.      |'
      write(*,*) '| Time-->frequency, X Unchanged            |'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*)
     +'bfxt  infile1  iopt   infile2  '
      write(*,*) ' '
      write(*,*) 'infile1 =name of X-T input file (space,time)'
      write(*,*) ' '
      write(*,*) ' iopt    1=forward transform (unwrap phase)'
      write(*,*) '         2=forward transform (no unwrap phase)'
      write(*,*) '        -1=inverse transform (two input files)'
      write(*,*) ' '
      write(*,*) 'infile1 =name of input file AMPLITUDES(iopt=-1 Only)'
      write(*,*) 'infile2 =name of input file PHASE(iopt=-1 Only)'
      write(*,*) ' '
      write(*,*) 'NOTE: Forward transform requires only one input file'
      write(*,*) '      Inverse transform requires two input files'
      write(*,*) ' '
      stop
      endif
      endif
c
      if(nargsx.lt.(2+karg)) then
      write(*,*) '  enter: name of input file1'
      read(*,'(a)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
      if(nargsx.lt.(3+karg)) then
      write(*,*) ' enter option: 1=forward (unwrap phase)'
      write(*,*) ' enter option: 2=forward (wrapped phase)'
      write(*,*) ' enter option:-1=inverse transform'
      read(*,*) iopt
      else
      call getarg(2,arg1)
      read(arg1,'(i5)') iopt
      endif
c
      if(nargsx.lt.(4+karg).and.iopt.eq.-1) then
      write(*,*) '  enter: name of input file2 (PHASE)'
      read(*,'(a)') infil2
      else
      call getarg(3,infil2)
      endif
      call namechk(12,infil2)
c
      return 
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,outlst,outfl2,outfl3,
     +infil,iopt,infil2,fsamin2,npr2)
      character*40      infil , outfl2, outfl3, outlst
      character*40      infil2
c
c...open output listing file...
      io11=11
      write(outlst,'(''bfxt.lst'')')
      open(io11,file=outlst,access='sequential')
      write(io11,*) '|---------------------------------|'
      write(io11,*) '| Copyright (C) 2017  P. Michaels |'
      write(io11,*) '|       All rights reserved       |'
      write(io11,*) '|see GNU General Public License   |'
      write(io11,*) '|---------------------------------|'
c
      write(io11,'('' Program bfxt  '')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfl2
      write(io11,'('' Output File: '',a)') outfl3
c
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''         iopt:'',i5)') iopt 
      if(iopt.ge.1) then
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Amplitude Output File: '',a)') outfl2
      write(io11,'('' Phase Output File: '',a)') outfl3
      else
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Input File2: '',a)') infil2
      write(io11,'('' Output File: '',a)') outfl2
      endif
c...decimate by iskip, scale to hz
      write(io11,'(''  '')')
      if(iopt.ge.1) then
      fnyq=float(npr2/2)*fsamin2*1000.
      write(io11,'(''  '')')
      write(io11,*) '----------------millihertz-----------------'
      write(io11,'(''  '')')
      write(io11,'('' Sample Interval= '',f10.5,'' millihertz'')')
     +fsamin2
      write(io11,'('' Nyquist Frequency= '',f10.5,'' millihertz'')')
     +fnyq/1000.
      write(io11,'(''  '')')
      write(io11,*) '----------------HZ-------------------------'
      write(io11,'(''  '')')
      write(io11,'('' Sample Interval= '',f10.3,'' Hz'')')
     +fsamin2*1000.
      write(io11,'('' Nyquist Frequency= '',f10.3,'' Hz'')') fnyq
      endif
      write(io11,'(''  '')')
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end

