c |$Id: bhed.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bhed.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c |   Loads headers with x,y,z and first break picks           |
c |                                                            |
c | Requires a ASCII file with source, receiver, and 1st breaks|
c |                                                            |
c | format: rec# pic sp sx sy sz vp rx ry rz gidb              |
c | rec# = trace number                                        |
c | sp   = shot point number                                   |
c | sx   = shot x-coord (Pos. East)                            |
c | sy   = shot y-coord (Pos. North)                           |
c | sz   = shot ground elev                                    |
c | vp   = receiver station number                             |
c | rx   = receiver x-coord                                    |
c | ry   = receiver y-coord                                    |
c | rz   = receiver ground elev                                |
c | pic  = pick time in seconds                                |
c | gidb = instrument gain                                     |
c | shtazi = shot polarization azimuth from north (degrees)    |
c | shtver = shot polarization from vertical (degrees)         |
c | geoazi = geophone polarization azimuth from north (deg.)   |
c | geover = geophone polarization from vertical (deg.)        |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
      real*4            s1(ndim)
      character*40      infil , outfil,  outlst, infil2
      character*4       aplc
c      integer*2         sp,vp
      character*4         sp,vp
      common /header/ hd
c
      integer*2         lowcut,highct,year,day,hour,minute,srec
      character*4       line,phone
      namelist /bhed/ lowcut,highct,line,year,day,hour,minute,phone,
     +sdepth,uphole,charge,srec

c...set some defaults
      lowcut=0
      highct=0
      year=2002
      day=0
      hour=0
      minute=0
      phone='vert'
      line='0001'
c
c...assign I/O units, initialize....
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''bhed'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +infil2,iupdn)
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,infil2,iupdn)               
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
      ntr=0
c
c...open geometry/pick file
      io12=12
      open(io12,file=infil2,access='sequential')
c
c...read namelist info
      if(iupdn.eq.0) then
      read(io12,bhed)
      write(io11,bhed)
      isvchg=charge
      endif
c
c...data loop=======================================....
c
      krec=0
      do 100 jrec=1,ntrace
c
      if(iupdn.eq.0) then
      read(io12,*,end=600,err=600) krec,pick,sp,sx,sy,sz,vp,rx,ry,rz,
     + kgidb,kshazi,kshver,kgeazi,kgever
      write(io11,543)
     +krec,pick,sp,sx,sy,sz,vp,rx,ry,rz,gidb,shtazi,shtver,
     +geoazi,geover
543   format(1x,i3,1x,f6.4,3x,a4,3(1x,f8.2),3x,a4,
     +3(1x,f8.2),i4,4(1x,i3.3))
      else
c                     else iupdn=1 download headers
      krec=krec+1
      endif
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,krec,iexit)
c
c...save vertical stack header
      nvshld=nvs
c
      if(iexit.eq.1) go to 600
      ntr = ntr + 1
c
c...clear headers
      if(iupdn.eq.0) then
      do 30 jh=1,240
   30 hd(jh)=0
c...note: (number of samples and sample interval saved
c         in arguments npts and fsamin)
c
c...assign header info
      charge=isvchg
      nvs=nvshld
      tic=1
      gaint=3
      gidb=kgidb
      shtazi=kshazi
      shtver=kshver
      geoazi=kgeazi
      geover=kgever
      tsnl=krec
      tsnt=krec
      write(spname,'(a4)') sp
      write(rstnam,'(a4)') vp
      espn=1
      tnofr=krec
ccc------------------
      mst=int(pick*1.E+06)
      cunits=1
      smul1=-1000
      smul2=-1000
      if(smul2.gt.0) then
      ishlo=sx/smul2
      ishla=sy/smul2
      irlo=rx/smul2
      irla=ry/smul2
      else
      ishlo=-sx*smul2
      ishla=-sy*smul2
      irlo=-rx*smul2
      irla=-ry*smul2
      endif
      if(smul1.gt.0) then
      ishe=sz/smul1
      irel=rz/smul1
      else
      ishe=-sz*smul1
      irel=-rz*smul1
      endif
      range=sqrt( (sx-rx)**2+(sy-ry)**2 )
      idist=range
c
      flc=lowcut
      fhc=highct
      lineid=line
      tyear=year
      tday=day
      thour=hour
      tmin=minute
      geoor=phone
      utimes=int(uphole*1000.+.5)
      if(smul1.gt.0) then
      ishd=int(sdepth/smul1)
      else
      ishd=int(-sdepth*smul1)
      endif
      ofrn=srec
c
c...output trace
      iunit2=2
      CALL BSEGOUT(iunit2,
     +icall2,npts,s1,fsamin,outfil,krec)
      else
c...end of insert mode **************************************
c                          else iupdn=1 
c...extract
c     krec=tsnl
c     krec=tsnt
      read(rstnam,'(a4)') vp
      read(spname,'(a4)') sp
ccc----------------------------
      pick=float(mst)/1.0E+06
      if (smul2.eq.0) then
      write(*,*) '  WARNING !!  smul2=0 reset smul2=1'
      smul2=1
      endif
      if(smul2.gt.0) then
      sx=float(smul2)*float(ishlo)
      sy=float(smul2)*float(ishla)
      rx=float(smul2)*float(irlo)
      ry=float(smul2)*float(irla)
      else
      sx=-float(ishlo)/float(smul2)
      sy=-ishla/float(smul2)
      rx=-irlo/float(smul2)
      ry=-irla/float(smul2)
      endif
      if(smul1.eq.0) then
      write(*,*) '  WARNING!!! smul1=0 reset smul1=1'
      smul1=1
      endif
      if(smul1.gt.0) then
      sz=float(smul1)*float(ishe)
      rz=float(smul1)*float(irel)
      else
      sz=-float(ishe)/float(smul1)
      rz=-float(irel)/float(smul1)
      endif
c
      if(ntr.eq.1) then
      lowcut=flc
      highct=fhc
      line=lineid
      year=tyear
      day=tday
      hour=thour
      minute=tmin
      phone=geoor
      uphole=float(utimes)/1000.
      if(smul1.gt.0) then
      sdepth=float(smul1)*float(ishd)
      else
      sdepth=-float(ishd)/float(smul1)
      endif
      srec=ofrn
      write(io12,bhed)
c     write(io12,900) lowcut,highct,year,day,line,hour,minute,sdepth,
c    +uphole,phone,srec
c 900 format(1x,'&bhed'/5x,'lowcut=',i4,' highct=',i4,' year=',i4,
c    +' day=',i2.2,','/5x,'line=''',a,''',' hour=',i2.2,' minute=',
c    +i2.2,','/
c    +5x,'sdepth=',f5.1,', uphole=',f6.4,', phone=''',a,''', srec=',
c    +i3.3,',',1x,'&end')
      endif
c
c...write header info
      write(io12,777) krec,pick,sp,sx,sy,sz,vp,rx,ry,rz,gidb,
     +shtazi,shtver,geoazi,geover
  777 format((1x,i3,1x,f6.4,3x,a4,3(1x,f9.3),3x,a4,
     +3(1x,f9.3),i4,4(1x,i3.3)))
      endif
c
c...display progress....
      call pltbar(bar,invbar,ibar,krec,ntrace)
c
100   continue      
c
c...END LOOP========================================....
600   continue
c
      write(*,'('' '')')
      if (iupdn .eq.0 ) then
      write(6,240) ntr,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
      else
      write(6,241) ntr,infil2,outlst
241   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)

      endif
c
c
      close(io11)
      close(io12)
      close(iunit1)
      close(iunit2)
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +infil2,iupdn)
c
      character*40      infil,infil2
      character*40      arg1,arg3
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bhed: Up or Down Loads header information|'
      write(*,*) '| From or To a text file.                  |'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*)
     +'bhed infile  header_file  iupdn  '
      write(*,*) ' '
      write(*,*) 'infile       =input file name '
      write(*,*) 'header_file  =file with selected header info '
      write(*,*) 'iupdn        =1 download headers to header_file'
      write(*,*) '             =0 upload headers to BSEGY data set'
      write(*,*) ' '
      stop
      endif
      endif
c
      if(nargsx.ge.(2+karg)) then
      call getarg(1,infil)
      else
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      endif
      call namechk(12,infil)
c
      if(nargsx.ge.(3+karg)) then
      call getarg(2,infil2)
      else
      write(*,*) '  enter: name of header file'
      read(*,'(a13)') infil2
      endif
c
      if(nargsx.ge.(4+karg)) then
      call getarg(3,arg3)
      read(arg3,'(i1)') iupdn
      else
      write(*,*) '  Select mode: 0=insert header data'
      write(*,*) '               1=download header data'
      read(*,*) iupdn
      endif
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,infil2,iupdn)               
      character*4       aplc
      character*40      outlst,outfil,infil,infil2
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*) '|---------------------------------|'
      write(io11,*) '| Copyright (C) 2017  P. Michaels |'
      write(io11,*) '|       All rights reserved       |'
      write(io11,*) '|see GNU General Public License   |'
      write(io11,*) '|---------------------------------|'
c
      write(io11,'('' Program bhed'')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Input Geometry/Pick File: '',a)') infil2
      write(io11,'('' Output File: '',a)') outfil
c
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''  iupdn= '',i5)')    iupdn     
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end

