// |$Id: onepole.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $       |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: onepole.c                                   |
   | DIRECTORY: bsu-2.0.2/src/sublibC4                          |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: one pole ARMA smoothing operator                  |
   |          pole on real axis in z-plane                      |
   |                                                            |
   |INPUTS:                                                     |
   |  s1     =input trace array                                 |
   |  npts   =number samples                                    |
   |  fsamin =sample interval in seconds                        |
   |  tcond  =time constant in seconds (amp=>1/e)               |
   |                                                            |
   |OUTPUTS: pointers                                           |
   |  s2     =output signal (filtered)                          |
   |                                                            |
   |  s2(j)=s1(j) + a*s2(j-1)   <===formula                     |
   |                                                            |
   |  where: a=exp(-alpha*fsamin) alpha=1/tcond                 |
   |                                                            |
   |     s2(t)=exp(-t/tcond) (*) s1(t) <==convolution           |
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <math.h>

      int onepole(int npts, float fsamin, float tcond,
         float s1[],float s2[])
     {

//...declare variables
      int               j;
      float             a,
                        alpha,
                        r;
//...tests of parameters
      if( (tcond==0.) | (fsamin==0.) | (npts<1) )
     {puts("ABORT: function onepole");
      puts("  invalid parameters");
      return(1);
     }

//...compute constants
//   (stable operator since use absolute values)

      alpha=1.00/fabs(tcond);      //decay constant
      r=exp(alpha*(float)fsamin);  //pole coord. on real axis
      a=1./r;                      //ARMA coefficient

//...apply operator
      s2[0]=s1[0];
      for (j=1;j<npts;j++) s2[j]=s1[j]+a*s2[j-1];

      return(0);
     } //  end onepole
